# -*- coding: utf-8 -*-
import logging
import logging.handlers
import os
from logging import handlers
import platform

from Dxr_mqtt.msg import msg
from Dxr_utils import gvalues
from Dxr_mqtt.dxr_mqtt import *

isPrintDebug = False
isPrintError = True
isPrintInfo = True


def setLogPrint(info=True, error=True, debug=False):
    global isPrintDebug, isPrintError, isPrintInfo
    isPrintInfo = info
    isPrintError = error
    isPrintDebug = debug


class Logger(object):
    #  日志级别关系映射
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'crit': logging.CRITICAL
    }

    def __init__(self, filename, level='info', when='D', backCount=3,
                 fmt='%(asctime)s line %(lineno)s %(levelname)s: %(message)s'):
        self.logger = logging.getLogger(filename)
        format_str = logging.Formatter(fmt)  # 设置日志格式
        self.logger.setLevel(self.level_relations.get(level))  # 设置日志级别
        th = handlers.TimedRotatingFileHandler(filename=filename, when=when, backupCount=backCount, encoding='utf-8')
        th.setFormatter(format_str)  # 设置文件里写入的格式
        # self.logger.addHandler(sh)  # 把对象加到logger里
        self.logger.addHandler(th)

    if platform.system() == 'Windows':
        if not os.path.exists('D:\\log'):  # 判断是否存在文件夹如果不存在则创建为文件夹
            try:
                os.makedirs('D:\\log')  # makedirs 创建文件时如果路径不存在会创建这个路径
            except Exception as ex:
                print(ex)
                pass
    else:
        if not os.path.exists(os.environ['HOME'] + '/log/'):  # 判断是否存在文件夹如果不存在则创建为文件夹
            try:
                os.makedirs(os.environ['HOME'] + '/log/')  # makedirs 创建文件时如果路径不存在会创建这个路径
            except Exception as ex:
                print(ex)
                pass


if platform.system() == 'Windows':
    all_log = Logger('D:\\log\\all.log', level='info')
    err_log = Logger('D:\\log\\error.log', level='error')
    debug_log = Logger('D:\\log\\debug.log', level='debug')
else:
    all_log = Logger(os.environ['HOME'] + '/log/all.log', level='info')
    err_log = Logger(os.environ['HOME'] + '/log/error.log', level='error')
    debug_log = Logger(os.environ['HOME'] + '/log/debug.log', level='debug')

all_publisher = Dxr_Publisher('/response/log_manager/' + gvalues.module_name + '/all')
debug_publisher = Dxr_Publisher('/response/log_manager/' + gvalues.module_name + '/debug')
error_publisher = Dxr_Publisher('/response/log_manager/' + gvalues.module_name + '/error')


def print_info(log_str):
    global isPrintInfo
    try:
        all_log.logger.info(log_str)
        m = msg()
        m.data = log_str
        m.status = True
        m.error = 0
        all_publisher.publish(m.get_json())
        if isPrintInfo:
            print(log_str)
    except Exception as ex:
        print(ex)


def print_debug(log_str):
    global isPrintDebug
    try:
        debug_log.logger.debug(log_str)
        all_log.logger.info(log_str)
        m = msg()
        m.data = log_str
        m.status = True
        m.error = 0
        debug_publisher.publish(m.get_json())
        if isPrintDebug:
            print(log_str)
    except Exception as ex:
        print(ex)


def print_error(log_str):
    global isPrintError
    try:
        all_log.logger.info(log_str)
        err_log.logger.error(log_str)
        m = msg()
        m.data = log_str
        m.status = True
        m.error = 0
        error_publisher.publish(m.get_json())
        if isPrintError:
            print(log_str)
    except Exception as ex:
        print(ex)



import struct

"""
用于进行bytes、bytearray、list数组相关操作
输出统一采用bytearray
"""

# crc16_modbus表
table_crc_hi = (
    0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
    0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
    0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
    0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
    0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1,
    0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41,
    0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1,
    0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
    0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
    0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40,
    0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1,
    0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
    0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
    0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40,
    0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
    0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
    0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
    0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
    0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
    0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
    0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
    0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40,
    0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1,
    0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
    0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
    0x80, 0x41, 0x00, 0xC1, 0x81, 0x40)

table_crc_lo = (
    0x00, 0xC0, 0xC1, 0x01, 0xC3, 0x03, 0x02, 0xC2, 0xC6, 0x06,
    0x07, 0xC7, 0x05, 0xC5, 0xC4, 0x04, 0xCC, 0x0C, 0x0D, 0xCD,
    0x0F, 0xCF, 0xCE, 0x0E, 0x0A, 0xCA, 0xCB, 0x0B, 0xC9, 0x09,
    0x08, 0xC8, 0xD8, 0x18, 0x19, 0xD9, 0x1B, 0xDB, 0xDA, 0x1A,
    0x1E, 0xDE, 0xDF, 0x1F, 0xDD, 0x1D, 0x1C, 0xDC, 0x14, 0xD4,
    0xD5, 0x15, 0xD7, 0x17, 0x16, 0xD6, 0xD2, 0x12, 0x13, 0xD3,
    0x11, 0xD1, 0xD0, 0x10, 0xF0, 0x30, 0x31, 0xF1, 0x33, 0xF3,
    0xF2, 0x32, 0x36, 0xF6, 0xF7, 0x37, 0xF5, 0x35, 0x34, 0xF4,
    0x3C, 0xFC, 0xFD, 0x3D, 0xFF, 0x3F, 0x3E, 0xFE, 0xFA, 0x3A,
    0x3B, 0xFB, 0x39, 0xF9, 0xF8, 0x38, 0x28, 0xE8, 0xE9, 0x29,
    0xEB, 0x2B, 0x2A, 0xEA, 0xEE, 0x2E, 0x2F, 0xEF, 0x2D, 0xED,
    0xEC, 0x2C, 0xE4, 0x24, 0x25, 0xE5, 0x27, 0xE7, 0xE6, 0x26,
    0x22, 0xE2, 0xE3, 0x23, 0xE1, 0x21, 0x20, 0xE0, 0xA0, 0x60,
    0x61, 0xA1, 0x63, 0xA3, 0xA2, 0x62, 0x66, 0xA6, 0xA7, 0x67,
    0xA5, 0x65, 0x64, 0xA4, 0x6C, 0xAC, 0xAD, 0x6D, 0xAF, 0x6F,
    0x6E, 0xAE, 0xAA, 0x6A, 0x6B, 0xAB, 0x69, 0xA9, 0xA8, 0x68,
    0x78, 0xB8, 0xB9, 0x79, 0xBB, 0x7B, 0x7A, 0xBA, 0xBE, 0x7E,
    0x7F, 0xBF, 0x7D, 0xBD, 0xBC, 0x7C, 0xB4, 0x74, 0x75, 0xB5,
    0x77, 0xB7, 0xB6, 0x76, 0x72, 0xB2, 0xB3, 0x73, 0xB1, 0x71,
    0x70, 0xB0, 0x50, 0x90, 0x91, 0x51, 0x93, 0x53, 0x52, 0x92,
    0x96, 0x56, 0x57, 0x97, 0x55, 0x95, 0x94, 0x54, 0x9C, 0x5C,
    0x5D, 0x9D, 0x5F, 0x9F, 0x9E, 0x5E, 0x5A, 0x9A, 0x9B, 0x5B,
    0x99, 0x59, 0x58, 0x98, 0x88, 0x48, 0x49, 0x89, 0x4B, 0x8B,
    0x8A, 0x4A, 0x4E, 0x8E, 0x8F, 0x4F, 0x8D, 0x4D, 0x4C, 0x8C,
    0x44, 0x84, 0x85, 0x45, 0x87, 0x47, 0x46, 0x86, 0x82, 0x42,
    0x43, 0x83, 0x41, 0x81, 0x80, 0x40)


# 格式符         C语言类型                 Python类型                Standard_size
# x             pad byte                no value                    -
# c	            char	                string of length 1          1
# b	            signed char	            integer	                    1
# B	            unsigned char	        integer	                    1
# ?	            _Bool	                bool	                    1
# h	            short	                integer	                    2
# H	            unsigned short          integer	                    2
# i	            int	                    integer	                    4
# I(大写的i)	    unsigned int	        integer	                    4
# l(小写的L)	    long                    integer	                    4
# L	            unsigned long	        long	                    4
# q	            long long	            long	                    8
# Q	            unsigned long long      long	                    8
# f	            float	                float	                    4
# d	            double	                float	                    8
# s	            char[]	                string                      -
# p	            char[]	                string                      -
# P	            void *	                long                        -


# Character	    Byte order	    Size	    Alignment
# @(默认)	    本机	            本机	        本机,凑够4字节
# =	            本机	            标准	        none,按原字节数
# <	            小端	            标准	        none,按原字节数
# >	            大端	            标准	        none,按原字节数
# !	            network(大端)	标准	        none,按原字节数

# region 显示功能
# 数组转16进制字符串
def bytes_to_hex_str(bs):
    """ 数组转16进制字符串
    :param bs: bytes、bytearray、list数组
    :return: 2位表示的16进制字符串
    """
    try:
        return ''.join(['%02X ' % b for b in bs])
    except Exception as e:
        print(e)


# 16进制字符串转bytearray数组
def hex_str_to_bytes(str_data):
    """ 16进制字符串转bytearray数组
    :param str_data: 16进制字符串
    :return: 对应bytearray数组
    """
    try:
        return bytearray.fromhex(str_data.replace(" ", ""))
    except Exception as e:
        print(e)


# endregion
# region 从数组获取数据
# 从bytes获取int8
def get_int8_from_bytes(bytes_data, index=0):
    """ 从bytes获取int8
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :return: 对应int8值
    """
    try:
        return struct.unpack("b", bytearray([bytes_data[index]]))[0]
    except Exception as e:
        print(e)


# 从bytes获取uint8
def get_uint8_from_bytes(bytes_data, index=0):
    """ 从bytes获取uint8
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :return: 对应uint8值
    """
    try:
        return int(struct.unpack("c", bytearray([bytes_data[index]]))[0][0])
    except Exception as e:
        print(e)


# 从bytes获取int16
def get_int16_from_bytes(bytes_data, index=0, order=""):
    """ 从bytes获取int16
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :param3 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应int16值
    """
    try:
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if (order is "") or (order.lower() == "little"):
            return struct.unpack("<h", bytes_data[index:index + 2])[0]
        elif order.lower() == "big":
            return struct.unpack(">h", bytes_data[index:index + 2])[0]
        else:
            print("order is not little or big")
    except Exception as e:
        print(e)


# 从bytes获取uint16
def get_uint16_from_bytes(bytes_data, index=0, order=""):
    """ 从bytes获取uint16
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :param3 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应uint16值
    """
    try:
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if (order is "") or (order.lower() == "little"):
            return struct.unpack("<H", bytes_data[index:index + 2])[0]
        elif order.lower() == "big":
            return struct.unpack(">H", bytes_data[index:index + 2])[0]
        else:
            print("order is not little or big")
    except Exception as e:
        print(e)


# 从bytes获取int32
def get_int32_from_bytes(bytes_data, index=0, order=""):
    """ 从bytes获取int32
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :param3 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应int32值
    """
    try:
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if (order is "") or (order.lower() == "little"):
            return struct.unpack("<i", bytes_data[index:index + 4])[0]
        elif order.lower() == "big":
            return struct.unpack(">i", bytes_data[index:index + 4])[0]
        else:
            print("order is not little or big")
    except Exception as e:
        print(e)


# 从bytes获取uint32
def get_uint32_from_bytes(bytes_data, index=0, order=""):
    """ 从bytes获取uint32
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :param3 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应uint32值
    """
    try:
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if (order is "") or (order.lower() == "little"):
            return struct.unpack("<I", bytes_data[index:index + 4])[0]
        elif order.lower() == "big":
            return struct.unpack(">I", bytes_data[index:index + 4])[0]
        else:
            print("order is not little or big")
    except Exception as e:
        print(e)


# 从bytes获取float
def get_float_from_bytes(bytes_data, index=0, order=""):
    """ 从bytes获取float
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :param3 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应float值
    """
    try:
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if (order is "") or (order.lower() == "little"):
            return struct.unpack("<f", bytes_data[index:index + 4])[0]
        elif order.lower() == "big":
            return struct.unpack(">f", bytes_data[index:index + 4])[0]
        else:
            print("order is not little or big")
    except Exception as e:
        print(e)


# 从bytes获取double
def get_double_from_bytes(bytes_data, index=0, order=""):
    """ 从bytes获取double
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 index: 所要读取的索引位置，默认为0
    :param3 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应double值
    """
    try:
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if (order is "") or (order.lower() == "little"):
            return struct.unpack("<d", bytes_data[index:index + 4])[0]
        elif order.lower() == "big":
            return struct.unpack(">d", bytes_data[index:index + 4])[0]
        else:
            print("order is not little or big")
    except Exception as e:
        print(e)


# 从bytes获取指定格式数据
def get_data_from_bytes(bytes_data, type="", index=0, order=""):
    """ 从bytes获取指定格式数据
    :param1 bytes_data: 所要读取的数组，bytes、bytearray、list类型
    :param2 type: 所要读取的数据格式，默认为“”，需输入字符串：int8、uint8、int16、uint16、int32、uint32、float、double，不区分大小写
    :param3 index: 所要读取的索引位置，默认为0
    :param4 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应值
    """
    if type.lower() == "":
        print("miss input: type")
        return None
    elif type.lower() == "int8":
        return get_int8_from_bytes(bytes_data, index)
    elif type.lower() == "uint8":
        return get_uint8_from_bytes(bytes_data, index)
    elif type.lower() == "int16":
        return get_int16_from_bytes(bytes_data, index, order)
    elif type.lower() == "uint16":
        return get_uint16_from_bytes(bytes_data, index, order)
    elif type.lower() == "int32":
        return get_int32_from_bytes(bytes_data, index, order)
    elif type.lower() == "float":
        return get_float_from_bytes(bytes_data, index, order)
    elif type.lower() == "double":
        return get_double_from_bytes(bytes_data, index, order)
    else:
        print("error input: type")
        return None


# endregion
# region 将数据加入数组
# 加入int8至bytes
def add_int8_to_bytes(bytes_data, value, index=None):
    """ 加入int8至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，int8类型
    :param3 index: 所要加入的索引位置，默认为最后
    :return: 对应bytearray结果
    """
    try:
        if value > 127 or value < -128:
            print("value is not between -128~127, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if index is not None:
            bytes_data[index] = struct.pack("b", value)[0]
        else:
            bytes_data.append(struct.pack("b", value)[0])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# 加入uint8至bytes
def add_uint8_to_bytes(bytes_data, value, index=None):
    """ 加入uint8至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，uint8类型
    :param3 index: 所要加入的索引位置，默认为最后
    :return: 对应bytearray结果
    """
    try:
        if value > 0xFF or value < 0:
            print("value is not between 0~255, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if index is not None:
            bytes_data[index] = struct.pack("b", value)[0]
        else:
            bytes_data.append(struct.pack("b", value)[0])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# 加入int16至bytes
def add_int16_to_bytes(bytes_data, value, index=None, order=""):
    """ 加入int16至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，int16类型
    :param3 index: 所要加入的索引位置，默认为最后
    :param4 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != int) or (value > 32767 or value < -32768):
            print("type of value or value is error, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        temp_bytes = bytearray()
        if (order is "") or (order.lower() == "little"):
            temp_bytes = struct.pack("<h", value)
        elif order.lower() == "big":
            temp_bytes = struct.pack(">h", value)
        else:
            print("order is not little or big")
        for x in range(2):
            if index is not None:
                bytes_data[index + x] = temp_bytes[x]
            else:
                bytes_data.append(temp_bytes[x])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# 加入uint16至bytes
def add_uint16_to_bytes(bytes_data, value, index=None, order=""):
    """ 加入uint16至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，uint16类型
    :param3 index: 所要加入的索引位置，默认为最后
    :param4 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != int) or (value > 65535 or value < 0):
            print("type of value is error, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        temp_bytes = bytearray()
        if (order is "") or (order.lower() == "little"):
            temp_bytes = struct.pack("<H", value)
        elif order.lower() == "big":
            temp_bytes = struct.pack(">H", value)
        else:
            print("order is not little or big")
        for x in range(2):
            if index is not None:
                bytes_data[index + x] = temp_bytes[x]
            else:
                bytes_data.append(temp_bytes[x])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# 加入int32至bytes
def add_int32_to_bytes(bytes_data, value, index=None, order=""):
    """ 加入int32至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，int32类型
    :param3 index: 所要加入的索引位置，默认为最后
    :param4 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if type(value) != int:
            print("type of value is error, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        temp_bytes = bytearray()
        if (order is "") or (order.lower() == "little"):
            temp_bytes = struct.pack("<i", value)
        elif order.lower() == "big":
            temp_bytes = struct.pack(">i", value)
        else:
            print("order is not little or big")
        for x in range(4):
            if index is not None:
                bytes_data[index + x] = temp_bytes[x]
            else:
                bytes_data.append(temp_bytes[x])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# 加入uint32至bytes
def add_uint32_to_bytes(bytes_data, value, index=None, order=""):
    """ 加入uint32至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，uint32类型
    :param3 index: 所要加入的索引位置，默认为最后
    :param4 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if type(value) != int:
            print("type of value is error, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        temp_bytes = bytearray()
        if (order is "") or (order.lower() == "little"):
            temp_bytes = struct.pack("<I", value)
        elif order.lower() == "big":
            temp_bytes = struct.pack(">I", value)
        else:
            print("order is not little or big")
        for x in range(4):
            if index is not None:
                bytes_data[index + x] = temp_bytes[x]
            else:
                bytes_data.append(temp_bytes[x])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# 加入float至bytes
def add_float_to_bytes(bytes_data, value, index=None, order=""):
    """ 加入float至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，float、int类型
    :param3 index: 所要加入的索引位置，默认为最后
    :param4 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != float) and (type(value) != int):
            print("type of value is error, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        temp_bytes = bytearray()
        if (order is "") or (order.lower() == "little"):
            temp_bytes = struct.pack("<f", float(value))
        elif order.lower() == "big":
            temp_bytes = struct.pack(">f", float(value))
        else:
            print("order is not little or big")
        for x in range(4):
            if index is not None:
                bytes_data[index + x] = temp_bytes[x]
            else:
                bytes_data.append(temp_bytes[x])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# 加入double至bytes
def add_double_to_bytes(bytes_data, value, index=None, order=""):
    """ 加入float至bytes
    :param1 bytes_data: 所要加入的数组，bytes、bytearray、list类型
    :param2 value: 所要加入的对象，double、int类型
    :param3 index: 所要加入的索引位置，默认为最后
    :param4 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != float) and (type(value) != int):
            print("type of value is error, input do not change")
            return bytes_data
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        temp_bytes = bytearray()
        if (order is "") or (order.lower() == "little"):
            temp_bytes = struct.pack("<d", float(value))
        elif order.lower() == "big":
            temp_bytes = struct.pack(">d", float(value))
        else:
            print("order is not little or big")
        for x in range(8):
            if index is not None:
                bytes_data[index + x] = temp_bytes[x]
            else:
                bytes_data.append(temp_bytes[x])
        return bytes_data
    except Exception as e:
        print(e)
        return bytes_data


# endregion
# region 将数据转换数组
# 获得int8对应bytearray
def get_int8_bytes(value):
    """ 获得int8对应bytearray
    :param1 value: 所要获取的对象，int8类型
    :return: 对应bytearray结果
    """
    try:
        if value > 127 or value < -128:
            print("value is not between 0~255, do not get bytes")
            return
        bytes_data = bytearray()
        bytes_data.append(struct.pack("b", value)[0])
        return bytes_data
    except Exception as e:
        print(e)
        return


# 获得uint8对应bytearray
def get_uint8_bytes(value):
    """ 获得uint8对应bytearray
    :param1 value: 所要获取的对象，uint8类型
    :return: 对应bytearray结果
    """
    try:
        if value > 0xFF or value < 0:
            print("value is not between 0~255, do not get bytes")
            return
        bytes_data = bytearray()
        bytes_data.append(value)
        return bytes_data
    except Exception as e:
        print(e)
        return


# 获得int16对应bytearray
def get_int16_bytes(value, order=""):
    """ 获得int16对应bytearray
    :param1 value: 所要获取的对象，int16类型
    :param2 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != int) or (value > 32767 or value < -32768):
            print("type of value or value is error, do not get bytes")
            return
        if (order is "") or (order.lower() == "little"):
            bytes_data = bytearray(struct.pack("<h", value))
        elif order.lower() == "big":
            bytes_data = bytearray(struct.pack(">h", value))
        else:
            print("order is not little or big")
            return
        return bytes_data
    except Exception as e:
        print(e)
        return


# 获得uint16对应bytearray
def get_uint16_bytes(value, order=""):
    """ 获得uint16对应bytearray
    :param1 value: 所要获取的对象，uint16类型
    :param2 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != int) or (value > 65535 or value < 0):
            print("type of value or value is error, do not get bytes")
            return
        if (order is "") or (order.lower() == "little"):
            bytes_data = bytearray(struct.pack("<H", value))
        elif order.lower() == "big":
            bytes_data = bytearray(struct.pack(">H", value))
        else:
            print("order is not little or big")
            return
        return bytes_data
    except Exception as e:
        print(e)
        return


# 获得int32对应bytearray
def get_int32_bytes(value, order=""):
    """ 获得int32对应bytearray
    :param1 value: 所要获取的对象，int32类型
    :param2 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != int) or (value > 2147483647 or value < -2147483648):
            print("type of value or value is error, do not get bytes")
            return
        if (order is "") or (order.lower() == "little"):
            bytes_data = bytearray(struct.pack("<i", value))
        elif order.lower() == "big":
            bytes_data = bytearray(struct.pack(">i", value))
        else:
            print("order is not little or big")
            return
        return bytes_data
    except Exception as e:
        print(e)
        return


# 获得uint32对应bytearray
def get_uint32_bytes(value, order=""):
    """ 获得uint32对应bytearray
    :param1 value: 所要获取的对象，uint32类型
    :param2 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != int) or (value > 4294967295 or value < 0):
            print("type of value or value is error, do not get bytes")
            return
        if (order is "") or (order.lower() == "little"):
            bytes_data = bytearray(struct.pack("<I", value))
        elif order.lower() == "big":
            bytes_data = bytearray(struct.pack(">I", value))
        else:
            print("order is not little or big")
            return
        return bytes_data
    except Exception as e:
        print(e)
        return


# 获得float对应bytearray
def get_float_bytes(value, order=""):
    """ 获得float对应bytearray
    :param1 value: 所要获取的对象，float、int类型
    :param2 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != float) and (type(value) != int):
            print("type of value or value is error, do not get bytes")
            return
        if (order is "") or (order.lower() == "little"):
            bytes_data = bytearray(struct.pack("<f", float(value)))
        elif order.lower() == "big":
            bytes_data = bytearray(struct.pack(">f", float(value)))
        else:
            print("order is not little or big")
            return
        return bytes_data
    except Exception as e:
        print(e)
        return


# 获得double对应bytearray
def get_double_bytes(value, order=""):
    """ 获得float对应bytearray
    :param1 value: 所要获取的对象，double、int类型
    :param2 order: 大小端，默认为小端，输入"big"或”little“
    :return: 对应bytearray结果
    """
    try:
        if (type(value) != float) and (type(value) != int):
            print("type of value or value is error, do not get bytes")
            return
        if (order is "") or (order.lower() == "little"):
            bytes_data = bytearray(struct.pack("<d", float(value)))
        elif order.lower() == "big":
            bytes_data = bytearray(struct.pack(">d", float(value)))
        else:
            print("order is not little or big")
            return
        return bytes_data
    except Exception as e:
        print(e)
        return


# endregion
# region 位功能
# 获得对应bit位的值
def get_bit_from_byte(target_byte, bit_index):
    """ 获得int8对应bytearray
    :param1 target_byte: 所要获取的对象所在字节，byte
    :param2 bit_index: 所要获取的对象所在字节位置，int类型
    :return: 对应bit结果, 0或1
    """
    try:
        if (type(target_byte) != int) or (target_byte > 255 or target_byte < 0):
            print("type of target_byte or target_byte is error, do not get bit")
            return
        if (type(bit_index) != int) or (bit_index > 7 or bit_index < 0):
            print("type of bit_index or bit_index is error, do not get bit")
            return
        return (target_byte >> bit_index) & 0x01
    except Exception as e:
        print(e)
        return


# 修改对应bit位的值
def add_bit_to_byte(target_byte, bit_index, bit_value):
    """ 获得int8对应bytearray
    :param1 target_byte: 所要获取的对象所在字节，byte
    :param2 bit_index: 所要获取的对象所在字节位置，int类型
    :param3 bit_value: 所要获取的对象所在字节位置，int类型
    :return: 对应byte结果
    """
    try:
        if (type(target_byte) != int) or (target_byte > 255 or target_byte < 0):
            print("type of target_byte or target_byte is error, do not get bit")
            return
        if (type(bit_index) != int) or (bit_index > 7 or bit_index < 0):
            print("type of bit_index or bit_index is error, do not get bit")
            return
        if (type(bit_value) != int) or (bit_value > 1 or bit_value < 0):
            print("type of bit_value or bit_value is error, do not get bit")
            return
        return target_byte | (bit_value << bit_index)
    except Exception as e:
        print(e)
        return


# 返回byte对应的bit二进制字符串
def get_bit_string_from_byte(target_byte):
    """ 返回byte对应的bit二进制字符串
    :param1 target_byte: 所要获取的对象字节，byte
    :return: 对应bit字符串
    """
    try:
        if (type(target_byte) != int) or (target_byte > 255 or target_byte < 0):
            print("type of target_byte or target_byte is error, do not get bit")
            return
        return format(0x0100 + result, 'b')[1:]
    except Exception as e:
        print(e)
        return


# endregion
# region crc16_modbus功能
# bytes加入crc16_modbus
def bytes_add_crc16_modbus(bytes_data, start_index=0, front=""):
    """ bytes加入crc16_modbus
    :param1 bytes_data: 所需加入CRC校验码的数组，bytes、bytearray、list类型
    :param2 start_index: 所需计算CRC校验码的开始索引，默认为0
    :param3 front: CRC校验码低位还是高位在前，默认低位在前，输入"low"或”high“
    :return: 对应bytearray结果
    """
    if type(bytes_data) != bytearray:
        bytes_data = bytearray(bytes_data)
    temp = crc16_modbus_bytes(bytes_data, start_index)
    if (front is "") or (front.lower() == "low"):
        bytes_data.append(temp[0])
        bytes_data.append(temp[1])
    elif front.lower() == "high":
        bytes_data.append(temp[1])
        bytes_data.append(temp[0])
    else:
        print("front is not low or high")
        return
    return bytes_data


# crc16_modbus返回int16
def crc16_modbus_int16(bytes_data, start_index=0):
    """ bytes加入crc16_modbus
    :param1 bytes_data: 所需加入CRC校验码的数组，bytes、bytearray、list类型
    :param2 start_index: 所需计算CRC校验码的开始索引，默认为0
    :return: CRC16数值 int16结果
    """
    crc_hi = crc_lo = 0xFF
    for ch in bytes_data[start_index:]:
        i = crc_hi ^ ch
        crc_hi = crc_lo ^ table_crc_hi[i]
        crc_lo = table_crc_lo[i]
    result = crc_hi << 8 | crc_lo
    print([crc_hi, crc_lo])
    return result


# crc16_modbus返回[crc_L,crc_H]
def crc16_modbus_bytes(bytes_data, start_index=0):
    """ bytes加入crc16_modbus
    :param1 bytes_data: 所需加入CRC校验码的数组，bytes、bytearray、list类型
    :param2 start_index: 所需计算CRC校验码的开始索引，默认为0
    :return: CRC16数值 bytearray结果[crc_L,crc_H]
    """
    crc_hi = crc_lo = 0xFF
    for ch in bytes_data[start_index:]:
        i = crc_hi ^ ch
        crc_hi = crc_lo ^ table_crc_hi[i]
        crc_lo = table_crc_lo[i]
    result = bytearray()
    result.append(crc_hi)
    result.append(crc_lo)
    return result


# endregion
# region 其他功能
# 返回帧头
def find_head(bytes_data, head_data):
    """ bytes加入crc16_modbus
    :param1 bytes_data: 所需寻找帧头的数组，bytes、bytearray、list类型
    :param2 head_data: 所需寻找的帧头，bytes、bytearray、list类型
    :return: 帧头的索引
    """
    try:
        if type(bytes_data) != bytearray:
            bytes_data = bytearray(bytes_data)
        if type(head_data) != bytearray:
            head_data = bytearray(head_data)
        num = len(head_data)
        for x in range(len(bytes_data) - num):
            if bytes_data[x:x + num] == head_data:
                return x
        return None
    except Exception as e:
        print(e)
        return None


# endregion

# 测试代码
if __name__ == '__main__':
    input_data = [0xF3, 0x03, 0x00, 0x01, 0x00, 0x01]
    print("*" * 50)
    # for x in range(8):
    #     print(get_bit_from_byte(0xf3, x))
    result = 0
    result = add_bit_to_byte(result, 2, 1)
    # 二进制显示result，8位
    print(get_bit_string_from_byte(result))
    print("*" * 50)
    pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Datas.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='Datas.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0b\x44\x61tas.proto\"\x16\n\x07Request\x12\x0b\n\x03msg\x18\x01 \x01(\t\"\x16\n\x05Reply\x12\r\n\x05\x64\x61tas\x18\x01 \x01(\x0c\x32\x31\n\nMainServer\x12#\n\tgetStream\x12\x08.Request\x1a\x06.Reply\"\x00(\x01\x30\x01\x62\x06proto3')
)




_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg', full_name='Request.msg', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=15,
  serialized_end=37,
)


_REPLY = _descriptor.Descriptor(
  name='Reply',
  full_name='Reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='datas', full_name='Reply.datas', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=39,
  serialized_end=61,
)

DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Reply'] = _REPLY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'Datas_pb2'
  # @@protoc_insertion_point(class_scope:Request)
  ))
_sym_db.RegisterMessage(Request)

Reply = _reflection.GeneratedProtocolMessageType('Reply', (_message.Message,), dict(
  DESCRIPTOR = _REPLY,
  __module__ = 'Datas_pb2'
  # @@protoc_insertion_point(class_scope:Reply)
  ))
_sym_db.RegisterMessage(Reply)



_MAINSERVER = _descriptor.ServiceDescriptor(
  name='MainServer',
  full_name='MainServer',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=63,
  serialized_end=112,
  methods=[
  _descriptor.MethodDescriptor(
    name='getStream',
    full_name='MainServer.getStream',
    index=0,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_REPLY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_MAINSERVER)

DESCRIPTOR.services_by_name['MainServer'] = _MAINSERVER

# @@protoc_insertion_point(module_scope)



from concurrent import futures
import re
import threading
import time
import cv2
import grpc
import base64
import numpy as np
from . import Datas_pb2
from . import Datas_pb2_grpc
import sys
import cv2
from . import global_values as gv


def setIP(ip):
    gv.ip = ip


def setPort(port):
    gv.port = port


def start_run():
    gv.isStart = True
    threading.Thread(target=run, daemon=True).start()


def run():
    # サーバーの宛先
    channel = grpc.insecure_channel(gv.ip + ':' + str(gv.port))
    stub = Datas_pb2_grpc.MainServerStub(channel)

    try:

        # リクエストデータを作成
        message = [Datas_pb2.Request(msg='give me the stream!!')]
        responses = stub.getStream(iter(message))

        for res in responses:
            # print(res.datas)

            # 画像を文字列などで扱いたい場合
            # b64d = base64.b64decode(res.datas)

            # バッファを作成
            dBuf = np.frombuffer(res.datas, dtype=np.uint8)

            # 作成したバッファにデータを入れる
            dst = cv2.imdecode(dBuf, cv2.IMREAD_COLOR)
            gv.video_que.queue.clear()
            gv.video_que.put(dst)
            if not gv.isStart:
                break

    except grpc.RpcError as e:
        gv.isStart = False
        print(e.details())


def get_frame():
    try:
        frame = gv.video_que.get(timeout=0.5)
        return 1, frame
    except Exception as ex:
        print(ex)
        return 0, None


def get_status():
    return gv.isStart


def start_get_frame_from_url_thread(url):
    if not gv.isStart_from_url:
        gv.isStart_from_url = True
        threading.Thread(target=start_get_frame_from_url, args=(url,), daemon=True).start()


def start_get_frame_from_url(url):
    cap = cv2.VideoCapture(gv.url)
    cap.set(cv2.CAP_PROP_FRAME_WIDTH, 640)
    cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 480)
    while gv.isStart_from_url:
        ret, frame = cap.read()
        gv.video_from_url_que.queue.clear()
        gv.video_from_url_que.put(frame)


def get_frame_from_url_status():
    return gv.isStart_from_url


def get_frame_from_url():
    try:
        frame = gv.video_from_url_que.get(timeout=0.5)
        return 1, frame
    except Exception as ex:
        print(ex)
        return 0, None


def stop_get_frame_from_url():
    gv.isStart_from_url = False



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
FTP常用操作
"""
from ftplib import FTP
import os


class FTP_OP(object):
    def __init__(self, host, port=2121, username="", password=""):
        """
            初始化ftp
        :param host: ftp主机ip
        :param username: ftp用户名
        :param password: ftp密码
        :param port:  ftp端口 （默认21）
        """
        self.host = host
        self.username = username
        self.password = password
        self.port = port

    def ftp_connect(self):
        """
        连接ftp
        :return:
        """
        ftp = FTP()
        ftp.set_debuglevel(0)  # 不开启调试模式
        ftp.connect(host=self.host, port=self.port)  # 连接ftp
        ftp.login(self.username, self.password)  # 登录ftp
        return ftp

    @staticmethod
    def download_file(url_, dst_file_path_):
        """
        从ftp下载文件到本地
        :param url_: ftp下载文件路径
        :param dst_file_path_: 本地存放路径
        :return:
        """
        ftp_arr = url_.split('//')[1].split('/')
        host_, port_ = ftp_arr[0].split(":")
        # print(host_, port_)
        ftp_file_path_ = ""
        for item in ftp_arr[1:]:
            ftp_file_path_ = ftp_file_path_ + "/" + item
        # print(ftp_file_path_)
        ftp_ = FTP_OP(host=host_, port=int(port_))
        buffer_size = 1024000  # 默认是8192
        ftp_servcie = ftp_.ftp_connect()
        print(ftp_servcie.getwelcome())  # 显示登录ftp信息
        ftp_file = ftp_file_path_
        write_file = dst_file_path_
        if not os.path.exists(write_file):
            open(write_file, 'w').close()
        with open(write_file, "wb") as f:
            ftp_servcie.retrbinary('RETR {0}'.format(ftp_file), f.write, buffer_size)
            f.close()
        ftp_servcie.quit()


if __name__ == '__main__':
    url = "ftp://192.168.10.102:2121/all.log"
    dst_file_path = os.environ['HOME'] + "/lu/all.log"
    FTP_OP.download_file(url, dst_file_path)


from Dxr_log.log import *
import threading
import time
import serial


class serial_class:
    # 初始化
    def __init__(self):
        self.obj_name = ""  # 对象名称
        self.port_name = ""  # 端口，GNU / Linux上的/ dev / ttyUSB0 等 或 Windows上的 COM3 等
        self.bps = 115200  # 波特率，标准值之一：50,75,110,134,150,200,300,600,1200,1800,2400,4800,9600,19200,38400,57600,115200
        self.time_out = 5  # 超时设置,None：永远等待操作，0为立即返回请求结果，其他值为等待超时时间(单位为秒）
        self.parity = ""  # N E O
        self.serial = {}  # 串口对象
        self.is_open = False  # 连接状态
        self.auto_connect_flag = True  # 内部重连标志
        self.callback_func = None  # 默认的回调函数
        self._auto_connect_thread = threading.Thread()  # 内部自动重连连接线程
        self._receive_thread = threading.Thread()  # 内部接收线程
        self.debug_flag = False  # debug模式

    # 串口连接 连接成功之后接收线程自动开启
    def connect(self):
        try:
            # 检查串口是否开启
            if self.is_open:
                self.close()
            self.is_open = False
            # 串口实例生成
            if not self.debug_flag:
                self.serial = serial.Serial(self.port_name, self.bps, timeout=self.time_out, parity=self.parity)
            self.is_open = True
            # 检查是否绑定接受回调
            if self.callback_func is None:
                print_error(self.obj_name + " " + "without dispose_func")
            print_info(self.obj_name + " " + "connect to " + self.port_name)
            # 内部接收线程开启
            self._receive_thread = threading.Thread(target=self._receive_thread_poc)
            self._receive_thread.daemon = True
            self._receive_thread.start()
            return True
        except Exception as e:
            print_error(self.obj_name + " " + "connect error:" + e.__str__())
            return False

    # 串口断开连接
    def close(self):
        try:
            # 关闭
            self.is_open = False
            time.sleep(0.5)
            if not self.debug_flag:
                self.serial.close()
            print_info(self.obj_name + " " + "close")
        except Exception as e:
            print_error(self.obj_name + " " + "close error:" + e.__str__())

    # 串口接收
    def read_msg(self):
        # 接收串口发送过来的数据
        if self.is_open:
            try:
                if not self.debug_flag:
                    if self.serial.in_waiting > 0:
                        data = self.serial.read(self.serial.in_waiting)
                        if self.callback_func is not None:
                            self.callback_func(data)
                        else:
                            print("receive:" + data)
                    else:
                        time.sleep(0.001)
            except Exception as e:
                raise e

    # 串口发送
    def send_msg(self, msg):
        try:
            if msg:
                # 检查串口状态
                if self.is_open:
                    if not self.debug_flag:
                        self.serial.write(msg)
                    else:
                        try:
                            print('serial debug send: '+''.join(['%02X ' % b for b in msg]))
                        except Exception as e:
                            print(e)
                else:
                    raise Exception("is not open")
        except Exception as e:
            # 非主动停止进行重启
            if self.is_open:
                print_debug(self.obj_name + " " + "send_msg Error:" + e.__str__())
                self._auto_connect()
            # 主动停止退出
            else:
                print_debug(self.obj_name + " " + "send fail and exit")

    # 自动重连线程
    def _auto_connect_thread_poc(self):
        print_info(self.obj_name + " " + "start auto_connect_thread_poc")
        while 1:
            try:
                # 检查初值
                if self.port_name != "" and self.bps != 0:
                    # 连接
                    if self.connect():
                        break
                    else:
                        # 连接失败则重复进行
                        time.sleep(5)
            except Exception as e:
                # 连接异常则重复进行
                print_error(self.obj_name + " " + "failed to connect to server: " + e.__str__())
                time.sleep(5)

    # 自动重连
    def _auto_connect(self):
        # 需要自动重连则开启自动重连线程
        if self.auto_connect_flag:
            # 检查是否重复开启
            if not self._auto_connect_thread.is_alive():
                self._auto_connect_thread = threading.Thread(target=self._auto_connect_thread_poc)
                self._auto_connect_thread.daemon = True
                self._auto_connect_thread.start()
            else:
                print_error(self.obj_name + " " + "auto_connect_thread has been started")
        # 不需要自动重连则断开连接
        else:
            self.close()

    # 接收线程
    def _receive_thread_poc(self):
        try:
            print_info(self.obj_name + " " + "start receive_thread")
            while self.is_open:
                self.read_msg()
            print_info("receive_thread normal exit")
        except Exception as e:
            # 中止属于程序主动执行，跳过重启
            if e.__str__().find("中止") > 0:
                pass
            else:
                # 非主动停止
                if self.is_open:
                    print_error(self.obj_name + " " + "receive_thread error:" + e.__str__())
                    self._auto_connect()


if __name__ == '__main__':
    # 接收回调方法
    def com_callback(data):
        print(data)


    # 生成串口实例
    com = serial_class()
    # 串口实例属性配置
    com.obj_name = "rs485_1"
    com.port_name = "COM4"
    com.bps = 9600
    com.time_out = 10
    com.parity = "N"
    com.auto_connect_flag = True
    com.callback_func = com_callback
    com.debug_flag = False
    # 串口连接
    com.connect()
    send_data = [0x01, 0x02, 0x03, 0x04]
    for x in range(10):
        # 串口发送
        com.send_msg(send_data)
        time.sleep(0.5)
    # 串口关闭
    com.close()



# -*- coding: utf-8 -*-
import json
import threading
import time

import paho.mqtt.client as mqtt

server_url = '127.0.0.1'
server_port = 1883
callback_dit = {}
publish_dit = []
mqttc = None
rc = -1
isPrintLog = False
client_id = str(int(time.time() * 1000))

topic_list_str_pub = ''
topic_list_str_sub = ''


def setServerUrl_2(url='127.0.0.1', port=1883, clientID=str(int(time.time() * 1000))):
    global server_url, server_port, client_id
    server_url = url
    server_port = port
    client_id = clientID


def setMqttLog_2(PrintLog=False):
    global isPrintLog
    isPrintLog = PrintLog


class Dxr_url_port:
    pass


def on_connect(client, userdata, flags, rrc):
    global rc
    rc = rrc
    print("Connected with result code " + str(rc))


# 一旦订阅到消息，回调此方法
def on_message(_, obj, msg):
    global sub_topic_dit, callback_dit
    topic = msg.topic
    callback_dit[topic](msg.payload)


def callback():
    pass


# 一旦订阅成功，回调此方法
def on_subscribe(mqttc, obj, mid, granted_qos):
    # print("Subscribed: " + str(mid) + " " + str(granted_qos))
    pass


# 一旦有log，回调此方法
def on_log(mqttc, obj, level, string):
    global isPrintLog
    if isPrintLog:
        print(string)


def Mqtt():
    global mqttc, server_url, server_port, client_id
    if mqttc is None:
        # 新建mqtt客户端，默认没有clientid，clean_session=True, transport="tcp"
        mqttc = mqtt.Client(clean_session=False, client_id=client_id)
        mqttc.will_set('/topic/'+client_id+'_2', '', retain=True)
        mqttc.on_message = on_message
        mqttc.on_connect = on_connect
        mqttc.on_subscribe = on_subscribe
        mqttc.on_log = on_log
        # 连接broker，心跳时间为60s
        mqttc.connect(server_url, server_port, 60)
        # 订阅该主题，QoS=0
        threading.Thread(target=mqttc.loop_forever).start()
    return mqttc


class Dxr_Subscriber_2:
    def __init__(self, topic, callback):
        global callback_dit, topic_list_str_pub, topic_list_str_sub
        self.mqttc = Mqtt()
        self.mqttc.subscribe(topic, qos=2)
        self.topic = topic
        self.callback = callback
        callback_dit[topic] = callback
        topic_list_str_sub = client_id + '_sub2:\n'
        for item in callback_dit.keys():
            topic_list_str_sub = '' + topic_list_str_sub + item + '\n'
        self.mqttc.publish('/topic/'+client_id+'_2', retain=True, payload=topic_list_str_pub + topic_list_str_sub)


class Dxr_Publisher_2:
    def __init__(self, topic):
        global publish_dit, topic_list_str_pub, topic_list_str_sub
        self.topic = topic
        # 新建mqtt客户端，默认没有clientid，clean_session=True, transport="tcp"
        self.mqttc = Mqtt()
        if topic not in publish_dit:
            publish_dit.append(topic)
        topic_list_str_pub = client_id + '_pub2:\n'
        for item in publish_dit:
            topic_list_str_pub = '' + topic_list_str_pub + item + '\n'
        self.mqttc.publish('/topic/'+client_id+'_2', retain=True, payload=topic_list_str_pub + topic_list_str_sub)

    def publish(self, msg):
        self.mqttc.publish(self.topic, json.dumps(msg), qos=2)


class Dxr_UnSubscriber_2:
    def __init__(self, topic):
        global callback_dit
        self.topic = topic
        self.mqttc = Mqtt()
        self.mqttc.unsubscribe(self.topic)
        callback_dit.pop(self.topic)



# coding=utf-8

import os
import platform
import time
from tkinter.messagebox import NO

import cv2
from .HCNetSDK import *
from .PlayCtrl import *
from time import sleep
from . import global_values as gv
Objdll = None
# 登录的设备信息
DEV_IP = None
DEV_PORT = 8000
DEV_USER_NAME = None
DEV_PASSWORD = create_string_buffer(b'Asb11023')

WINDOWS_FLAG = True
win = None  # 预览窗口
funcRealDataCallBack_V30 = None  # 实时预览回调函数，需要定义为全局的
PlayCtrl_Port = c_long(-1)  # 播放句柄
Playctrldll = None  # 播放库
FuncDecCB = None   # 播放库解码回调函数，需要定义为全局的

# 获取当前系统环境
def GetPlatform():
    sysstr = platform.system()
    print('' + sysstr)
    if sysstr != "Windows":
        global WINDOWS_FLAG
        WINDOWS_FLAG = False

# 设置SDK初始化依赖库路径
def SetSDKInitCfg():
    global Objdll
    # 设置HCNetSDKCom组件库和SSL库加载路径
    # print(os.getcwd())
    if WINDOWS_FLAG:
        strPath = os.getcwd().encode('gbk')
        sdk_ComPath = NET_DVR_LOCAL_SDK_PATH()
        sdk_ComPath.sPath = strPath
        Objdll.NET_DVR_SetSDKInitCfg(2, byref(sdk_ComPath))
        Objdll.NET_DVR_SetSDKInitCfg(3, create_string_buffer(strPath + b'\libcrypto-1_1-x64.dll'))
        Objdll.NET_DVR_SetSDKInitCfg(4, create_string_buffer(strPath + b'\libssl-1_1-x64.dll'))
    else:
        strPath = os.getcwd().encode('utf-8')
        sdk_ComPath = NET_DVR_LOCAL_SDK_PATH()
        sdk_ComPath.sPath = strPath
        Objdll.NET_DVR_SetSDKInitCfg(2, byref(sdk_ComPath))
        Objdll.NET_DVR_SetSDKInitCfg(3, create_string_buffer(strPath + b'/libcrypto.so.1.1'))
        Objdll.NET_DVR_SetSDKInitCfg(4, create_string_buffer(strPath + b'/libssl.so.1.1'))

def LoginDev(Objdll):
    # 登录注册设备
    device_info = NET_DVR_DEVICEINFO_V30()
    lUserId = Objdll.NET_DVR_Login_V30(DEV_IP, DEV_PORT, DEV_USER_NAME, DEV_PASSWORD, byref(device_info))
    return (lUserId, device_info)

def DecCBFun(nPort, pBuf, nSize, pFrameInfo, nUser, nReserved2):
    # 解码回调函数
    if pFrameInfo.contents.nType == 3:
        # print('get frame' + str(time.time()))
        # 解码返回视频YUV数据，将YUV数据转成jpg图片保存到本地
        # 如果有耗时处理，需要将解码数据拷贝到回调函数外面的其他线程里面处理，避免阻塞回调导致解码丢帧
        sFileName = ('../../pic/test_stamp.jpg')
        nWidth = pFrameInfo.contents.nWidth
        nHeight = pFrameInfo.contents.nHeight
        nType = pFrameInfo.contents.nType
        dwFrameNum = pFrameInfo.contents.dwFrameNum
        nStamp = pFrameInfo.contents.nStamp
        # print(nWidth, nHeight, nType, dwFrameNum, nStamp, sFileName)

        lRet = Playctrldll.PlayM4_ConvertToJpegFile(pBuf, nSize, nWidth, nHeight, nType, c_char_p(sFileName.encode()))
        if lRet == 0:
            print('PlayM4_ConvertToJpegFile fail, error code is:', Playctrldll.PlayM4_GetLastError(nPort))
        else:
            print('PlayM4_ConvertToJpegFile success' + str(time.time()))
            gv.video_que.queue.clear()
            gv.video_que.put(cv2.imread(sFileName))

def RealDataCallBack_V30(lPlayHandle, dwDataType, pBuffer, dwBufSize, pUser):
    # 码流回调函数
    if dwDataType == NET_DVR_SYSHEAD:
        # 设置流播放模式
        Playctrldll.PlayM4_SetStreamOpenMode(PlayCtrl_Port, 0)
        # 打开码流，送入40字节系统头数据
        if Playctrldll.PlayM4_OpenStream(PlayCtrl_Port, pBuffer, dwBufSize, 1024*1024):
            # 设置解码回调，可以返回解码后YUV视频数据
            global FuncDecCB
            FuncDecCB = DECCBFUNWIN(DecCBFun)
            Playctrldll.PlayM4_SetDecCallBackExMend(PlayCtrl_Port, FuncDecCB, None, 0, None)
            # 开始解码播放
            if Playctrldll.PlayM4_Play(PlayCtrl_Port, None):
                print(u'播放库播放成功')
            else:
                print(u'播放库播放失败')
        else:
            print(u'播放库打开流失败')
    elif dwDataType == NET_DVR_STREAMDATA:
        Playctrldll.PlayM4_InputData(PlayCtrl_Port, pBuffer, dwBufSize)
    else:
        print (u'其他数据,长度:', dwBufSize)

def OpenPreview(Objdll, lUserId, callbackFun):
    '''
    打开预览
    '''
    preview_info = NET_DVR_PREVIEWINFO()
    preview_info.hPlayWnd = 0
    preview_info.lChannel = 1  # 通道号
    preview_info.dwStreamType = 0  # 主码流
    preview_info.dwLinkMode = 0  # TCP
    preview_info.bBlocked = 1  # 阻塞取流

    # 开始预览并且设置回调函数回调获取实时流数据
    lRealPlayHandle = Objdll.NET_DVR_RealPlay_V40(lUserId, byref(preview_info), callbackFun, None)
    return lRealPlayHandle

def InputData(fileMp4, Playctrldll):
    while True:
        pFileData = fileMp4.read(4096)
        if pFileData is None:
            break

        if not Playctrldll.PlayM4_InputData(PlayCtrl_Port, pFileData, len(pFileData)):
            break

def start_hik_sdk():
    global Objdll, PlayCtrl_Port, Playctrldll, DEV_IP,  DEV_USER_NAME
    DEV_IP = create_string_buffer(bytes(gv.hik_ip, encoding='utf-8'))
    DEV_PASSWORD = create_string_buffer(bytes(gv.hik_password, encoding='utf-8'))
    DEV_USER_NAME = create_string_buffer(bytes(gv.hik_username, encoding='utf-8'))
    
    # 获取系统平台
    GetPlatform()
    path = os.path.abspath(__file__)
    path = path.replace('test_main.py', '')
    path += 'lib/linux'
    print(path)
    os.chdir(path)
    Objdll = cdll.LoadLibrary(r'./libhcnetsdk.so')
    Playctrldll = cdll.LoadLibrary(r'./libPlayCtrl.so')
    SetSDKInitCfg()  # 设置组件库和SSL库加载路径
    # 初始化DLL
    Objdll.NET_DVR_Init()
    # 启用SDK写日志
    Objdll.NET_DVR_SetLogToFile(3, bytes('./SdkLog_Python/', encoding="utf-8"), False)
    # 获取一个播放句柄
    if not Playctrldll.PlayM4_GetPort(byref(PlayCtrl_Port)):
        print(u'获取播放库句柄失败')

    # 登录设备
    (lUserId, device_info) = LoginDev(Objdll)
    if lUserId < 0:
        err = Objdll.NET_DVR_GetLastError()
        print('Login device fail, error code is: %d' % Objdll.NET_DVR_GetLastError())
        # 释放资源
        Objdll.NET_DVR_Cleanup()
        exit()

    # 定义码流回调函数
    funcRealDataCallBack_V30 = REALDATACALLBACK(RealDataCallBack_V30)
    # 开启预览
    lRealPlayHandle = OpenPreview(Objdll, lUserId, funcRealDataCallBack_V30)
    if lRealPlayHandle < 0:
        print ('Open preview fail, error code is: %d' % Objdll.NET_DVR_GetLastError())
        # 登出设备
        Objdll.NET_DVR_Logout(lUserId)
        # 释放资源
        Objdll.NET_DVR_Cleanup()
        exit()

    #show Windows
    # win.mainloop()
    while gv.is_hk_start:
        time.sleep(1)
    # # 开始云台控制
    # lRet = Objdll.NET_DVR_PTZControl(lRealPlayHandle, PAN_LEFT, 0)
    # if lRet == 0:
    #     print ('Start ptz control fail, error code is: %d' % Objdll.NET_DVR_GetLastError())
    # else:
    #     print ('Start ptz control success')

    # # 转动一秒
    # sleep(1)

    # # 停止云台控制
    # lRet = Objdll.NET_DVR_PTZControl(lRealPlayHandle, PAN_LEFT, 1)
    # if lRet == 0:
    #     print('Stop ptz control fail, error code is: %d' % Objdll.NET_DVR_GetLastError())
    # else:
    #     print('Stop ptz control success')

    # 关闭预览
    Objdll.NET_DVR_StopRealPlay(lRealPlayHandle)

    # 停止解码，释放播放库资源
    if PlayCtrl_Port.value > -1:
        Playctrldll.PlayM4_Stop(PlayCtrl_Port)
        Playctrldll.PlayM4_CloseStream(PlayCtrl_Port)
        Playctrldll.PlayM4_FreePort(PlayCtrl_Port)
        PlayCtrl_Port = c_long(-1)

    # 登出设备
    Objdll.NET_DVR_Logout(lUserId)

    # 释放资源
    Objdll.NET_DVR_Cleanup()




# -*- coding: utf-8 -*-
import copy
import inspect
import json
from multiprocessing import RLock
import os
import sys
import threading
import time
from loguru import logger
from tabulate import tabulate

#  时间输出格式为：2020-01-01 00:00:00
logid = logger.add("run.log", 
           rotation="1 days",
           retention="7 days",
           format="{time:YYYY-MM-DD HH:mm:ss} | {level:<8} |{message}",
           enqueue=True,
           catch=True,
           level="DEBUG")
logger.remove(0)
logger.add(sys.stdout, 
           format="{time:YYYY-MM-DD HH:mm:ss} | <blue><bold>{level:<8}</bold></blue> | {message}",
           colorize=True,
           enqueue=True,
           catch=True,
           level="DEBUG")

# 添加错误日志输出到error.log
logger.add("error.log", 
        rotation="1 days",
        retention="7 days",
        format="{time:YYYY-MM-DD HH:mm:ss} | {level} | {message}",
        enqueue=True,
        catch=True,
        level="ERROR")


def set_log_level(level):
    global logid
    logger.remove(logid)
    # 配置终端输出按照level级别过滤后输出
    my_filter = lambda record: record["level"].name == level
    logid = logger.add("run.log", 
           rotation="1 days",
           retention="7 days",
           format="{time:YYYY-MM-DD HH:mm:ss} {level} {message}",
           colorize=False,
           enqueue=True,
           catch=True,
           level="DEBUG",
           filter=my_filter)

import paho.mqtt.client as mqtt

server_url = '127.0.0.1'
server_port = 1883
callback_dit = {}
publish_dit = []
mqttc = None
rc = -1
isPrintLog = False
client_id = str(int(time.time() * 1000))
topic_list_str_pub = ''
topic_list_str_sub = ''
isReconnect = False
mqtt_thread = None
heart_beat_time = 5
sn = None


def setServerUrl(url='127.0.0.1', port=1883, clientID=str(int(time.time() * 1000))):
    global server_url, server_port, client_id
    server_url = url
    server_port = int(port)
    client_id = clientID


def setSn(serialNumber):
    global sn
    sn = serialNumber


def setHeartBeatTime(heart_time=5):
    global heart_beat_time
    heart_beat_time = heart_time


def setMqttLog(PrintLog=False):
    global isPrintLog
    isPrintLog = PrintLog


class Dxr_url_port:
    pass


def get_cond(topic):
    global callback_dit
    if not isinstance(topic, str):
        topic = topic.topic
    if topic not in callback_dit:
        Dxr_Subscriber(topic, None)
    return callback_dit[topic]['cond']


def on_connect(client, userdata, flags, rrc):
    global rc, isReconnect, mqttc
    rc = rrc
    print("Connected with result code " + str(rc))
    if isReconnect:
        isReconnect = False
        # 获取callback_dit中的所有topic,键
        keys = callback_dit.keys()
        # 重新订阅
        for item in keys:
            client.unsubscribe(item)
            client.subscribe(item, 0)


# 一旦订阅到消息，回调此方法
def on_message(client, obj, msg):
    global callback_dit
    topic = msg.topic
    try:
        threading.Thread(target=callback, args=(topic, msg, client), daemon=True).start()
        # callback(topic, msg, client)
    except Exception as ex:
        print(ex)
        keys = callback_dit.keys()
        topic_arr = topic.split("/")[1:]
        for item in keys:
            item_arr = item.split("/")[1:]
            isThisTopic = True
            if len(item_arr) == len(topic_arr):
                for i in range(len(item_arr)):
                    if item_arr[i] == "+":
                        continue
                    if item_arr[i] != topic_arr[i]:
                        isThisTopic = False
                        break
            if isThisTopic:
                threading.Thread(target=callback, args=(item, msg, client), daemon=True).start()
                # callback(item, msg, topic)
                break


def callback(topic, msg, client):
    global callback_dit
    msg = str(msg.payload.decode("utf-8"))
    if type(msg) is str:
        msg = json.loads(msg)
    if topic not in callback_dit:
        isTongPei = False
        keys = callback_dit.keys()
        for item in keys:
            if "+" in item:
                item_arr = item.split("/")[1:]
                topic_arr = topic.split("/")[1:]
                if item_arr[0] == topic_arr[0]:
                    isTongPei = True
                    break
        if not isTongPei:
            callback_dit[topic] = {'func': None, 'cond': threading.Event(), 'msg': None}
    callback_dit[topic]['msg'] = msg
    cond = get_cond(topic)
    cond.set()
    cond.clear()
    if callback_dit[topic]['func'] is not None:
        func = callback_dit[topic]['func']
        if '+' in topic:
            func(msg, client)
        else:
            func(msg, topic)


# 一旦订阅成功，回调此方法
def on_subscribe(mqttc, obj, mid, granted_qos):
    # print("Subscribed: " + str(mid) + " " + str(granted_qos))
    pass


# 一旦有log，回调此方法
def on_log(mqttc, obj, level, string):
    global isPrintLog
    if isPrintLog:
        print(string)


def on_disconnect(client, userdata, rrc):
    global rc, isReconnect, mqttc
    rc = rrc
    isReconnect = True
    print("Disconnected with result code " + str(rc))


def Mqtt():
    global mqttc, server_url, server_port, client_id, mqtt_thread
    if mqttc is None:
        try:
            # 新建mqtt客户端，默认没有clientid，clean_session=True, transport="tcp"
            mqttc = mqtt.Client(client_id=client_id)
            mqttc.will_set('/topic/' + client_id, '', retain=True)
            mqttc.on_message = on_message
            mqttc.on_connect = on_connect
            mqttc.on_subscribe = on_subscribe
            mqttc.on_disconnect = on_disconnect
            mqttc.on_log = on_log
            # 连接broker，心跳时间为60s
            mqttc.connect(server_url, server_port, heart_beat_time)
            # 订阅该主题，QoS=0
            mqtt_thread = threading.Thread(target=mqttc.loop_forever, daemon=True)
            mqtt_thread.start()
            return mqttc
        except Exception as ex:
            print(ex)
            return None
    else:
        return mqttc


def get_mqtt_status():
    global mqttc
    if mqttc is None:
        return False
    else:
        if mqttc.is_connected():
            return True
        else:
            return False


class Dxr_Subscriber:
    def __init__(self, topic, callback):
        global callback_dit, topic_list_str_pub, topic_list_str_sub, mqttc, sn
        if mqttc is None:
            mqttc = Mqtt()
        if isinstance(topic, str):
            self.topic = topic
        else:
            self.topic = topic.topic
        if sn is not None and sn not in self.topic:
            self.topic = f'/{sn}/client{self.topic}'
        mqttc.subscribe(self.topic)
        self.callback = callback
        if self.topic not in callback_dit:
            cond = threading.Event()
            callback_dit[self.topic] = {'func': self.callback, 'cond': cond, 'msg': None}
        else:
            callback_dit[self.topic]['func'] = self.callback
        topic_list_str_sub = client_id + '_sub:\n'
        for item in callback_dit.keys():
            topic_list_str_sub = '' + topic_list_str_sub + item + '\n'
        if mqttc is not None:
            if mqttc.is_connected():
                mqttc.publish('/topic/' + client_id, retain=True, payload=topic_list_str_pub + topic_list_str_sub)


# 定义一个订阅者类的解释器
class dxr_subscriber:
    def __init__(self, msg):
        global sn
        # 如果msg的类型是str,则将msg赋值给self.topic
        if isinstance(msg, str):
            self.topic = msg
        else:
            self.topic = msg.topic
        self.func = None

    def __call__(self, func):
        # 将函数绑定到解释器上
        self.func = func
        Dxr_Subscriber(self.topic, self.func)


class Dxr_Publisher:
    def __init__(self, topic):
        global publish_dit, topic_list_str_pub, topic_list_str_sub, callback_dit, mqttc, sn
        if mqttc is None:
            mqttc = Mqtt()
        if isinstance(topic, str):
            self.topic = topic
            self.data_type = None
        else:
            self.topic = topic.topic
            self.data_type = topic
        if sn is not None and sn not in self.topic:
            self.topic = f'/{sn}/pc{self.topic}'
        if self.topic not in publish_dit:
            publish_dit.append(self.topic)
        topic_list_str_pub = client_id + '_pub:\n'
        for item in publish_dit:
            topic_list_str_pub = '' + topic_list_str_pub + item + '\n'
        if mqttc is not None:
            if mqttc.is_connected():
                mqttc.publish('/topic/' + client_id, retain=True, payload=topic_list_str_pub + topic_list_str_sub)

    def publish(self, msg):
        global mqttc
        if mqttc is None:
            mqttc = Mqtt()
        try:
            if self.data_type:
                if mqttc is not None:
                    if mqttc.is_connected():
                        # print(f'publish:{self.topic}, msg:{msg.get_json()}')
                        mqttc.publish(self.topic, msg.get_json(), qos=0)
            else:
                if mqttc is not None:
                    if mqttc.is_connected():
                        mqttc.publish(self.topic, json.dumps(msg), qos=0)
        except Exception as ex:
            print(f'publish error: {ex}')

    '''
    使用await_publish来实现消息闭环
    await_publish(msg, timeout, topic)
    {
        msg 为发送的消息
        timeout 不指定的时候，为一直等待设定话题的消息，指定后超时时间中未收到消息，会接收到一个None消息
        topic 为指定闭环消息的话题类型，如果不指定，则默认为原话题类型后追加'_response',可以传递字符串类型的话题，也可以传递消息类型
    }
    '''

    def await_publish(self, msg, timeout=None, topic=None):
        global callback_dit, mqttc
        if mqttc is None:
            mqttc = Mqtt()
        if self.data_type:
            if mqttc is not None:
                if mqttc.is_connected():
                    mqttc.publish(self.topic, msg.get_json(), qos=0)
        else:
            if mqttc is not None:
                if mqttc.is_connected():
                    mqttc.publish(self.topic, json.dumps(msg), qos=0)
        await_topic = None
        if topic is None:
            await_topic = self.topic + '_response'
        else:
            if isinstance(topic, str):
                await_topic = topic
            else:
                await_topic = topic.topic
        if await_topic is None:
            return None
        await_topic = await_topic.replace('pc', 'client')
        cond = get_cond(await_topic)
        if cond is None:
            return None
        if timeout is None:
            cond.wait()
        else:
            cond.wait(timeout)
        return callback_dit[await_topic]['msg']

    def stop_awit_publish(self, topic=None):
        global callback_dit, mqttc
        if topic is None:
            await_topic = self.topic + '_response'
        else:
            if isinstance(topic, str):
                await_topic = topic
            else:
                await_topic = topic.topic
        if await_topic is None:
            return None
        await_topic = await_topic.replace('pc', 'client')
        cond = get_cond(await_topic)
        if cond is None:
            return None
        cond.set()
        cond.clear()

class Dxr_UnSubscriber:
    def __init__(self, topic):
        global callback_dit, mqttc
        if mqttc is None:
            mqttc = Mqtt()
        self.topic = topic
        mqttc.unsubscribe(self.topic)
        callback_dit.pop(self.topic)

# 定义一个猴子补丁，用来替换原来的print函数
def monky_print(*args, **kwargs):
    tmp_args = copy.deepcopy(args)
    # 获取所在文件名和行号
    file_name = inspect.stack()[1].filename
    # 只取文件名
    file_name = os.path.basename(file_name)
    line = inspect.stack()[1].lineno
    # 获取当前的线程名
    thread_name = threading.current_thread().name
    # 获取当前线程数量
    thread_count = threading.active_count()
    isException = False
    # 判断*args是否是错误类型
    for arg in args:
        if isinstance(arg, Exception):
            isException = True
            break
    if isException:
        logger.opt(colors=True).exception(f'<yellow>{file_name}:{line}</yellow>(<blue>{thread_name}:{thread_count}</blue>) {tmp_args}')
    else:
        # for arg in args:
        #     logger.info(arg)
        # # # 将args转换为字符串
        # args = [str(arg) for arg in args]
        # # # 将字符串拼接起来
        # args = " ".join(args)
        # 如果**kwargs中有end参数，则使用end参数的值，否则使用默认值\n
        end = kwargs.get("end", " ")
        # 将end格式添加到args中
        final_str = ''
        for arg in tmp_args:
            try:
                if isinstance(arg, (dict, tuple, list)):
                    # 如果不是list，转换为list
                    if not isinstance(arg, list):
                        arg = list(arg.items())
                    final_str = final_str + '\r\n' + tabulate(arg, maxcolwidths=30, tablefmt='grid', showindex=True, headers='keys') + end
                else:
                    final_str = final_str + str(arg) + end
            except:
                final_str = final_str + str(arg) + end
        logger.opt(colors=True).info(f'<yellow>{file_name}:{line}</yellow>(<blue>{thread_name}:{thread_count}</blue>)\r\n{final_str}')
    
        
# print = monky_print


# coding=utf-8

import ctypes
import sys
from ctypes import *

# 回调函数类型定义

if 'linux' in sys.platform:
    fun_ctype = CFUNCTYPE
else:
    fun_ctype = WINFUNCTYPE

# 云台控制命令
LIGHT_PWRON = 2  #接通灯光电源
WIPER_PWRON = 3  #接通雨刷开关
FAN_PWRON = 4  #接通风扇开关
HEATER_PWRON = 5  #接通加热器开关
AUX_PWRON1 = 6  #接通辅助设备开关
AUX_PWRON2 = 7  #接通辅助设备开关
ZOOM_IN = 11  #焦距变大(倍率变大)
ZOOM_OUT = 12  #焦距变小(倍率变小)
FOCUS_NEAR = 13  #焦点前调
FOCUS_FAR = 14  #焦点后调
IRIS_OPEN = 15  #光圈扩大
IRIS_CLOSE = 16  #光圈缩小
TILT_UP = 21  #云台上仰
TILT_DOWN = 22  #云台下俯
PAN_LEFT = 23  #云台左转
PAN_RIGHT = 24  #云台右转
UP_LEFT = 25  #云台上仰和左转
UP_RIGHT = 26  #云台上仰和右转
DOWN_LEFT = 27  #云台下俯和左转
DOWN_RIGHT = 28  #云台下俯和右转
PAN_AUTO = 29  #云台左右自动扫描
TILT_DOWN_ZOOM_IN  = 58  #云台下俯和焦距变大(倍率变大)
TILT_DOWN_ZOOM_OUT = 59  #云台下俯和焦距变小(倍率变小)
PAN_LEFT_ZOOM_IN = 60  #云台左转和焦距变大(倍率变大)
PAN_LEFT_ZOOM_OUT = 61  #云台左转和焦距变小(倍率变小)
PAN_RIGHT_ZOOM_IN = 62  #云台右转和焦距变大(倍率变大)
PAN_RIGHT_ZOOM_OUT = 63  #云台右转和焦距变小(倍率变小)
UP_LEFT_ZOOM_IN = 64  #云台上仰和左转和焦距变大(倍率变大)
UP_LEFT_ZOOM_OUT = 65  #云台上仰和左转和焦距变小(倍率变小)
UP_RIGHT_ZOOM_IN = 66  #云台上仰和右转和焦距变大(倍率变大)
UP_RIGHT_ZOOM_OUT = 67  #云台上仰和右转和焦距变小(倍率变小)
DOWN_LEFT_ZOOM_IN = 68  #云台下俯和左转和焦距变大(倍率变大)
DOWN_LEFT_ZOOM_OUT = 69  #云台下俯和左转和焦距变小(倍率变小)
DOWN_RIGHT_ZOOM_IN  = 70  #云台下俯和右转和焦距变大(倍率变大)
DOWN_RIGHT_ZOOM_OUT = 71  #云台下俯和右转和焦距变小(倍率变小)
TILT_UP_ZOOM_IN = 72  #云台上仰和焦距变大(倍率变大)
TILT_UP_ZOOM_OUT = 73  #云台上仰和焦距变小(倍率变小)

# 码流回调数据类型
NET_DVR_SYSHEAD = 1
NET_DVR_STREAMDATA = 2
NET_DVR_AUDIOSTREAMDATA = 3
NET_DVR_PRIVATE_DATA = 112

# 设备参数结构体 V30
class NET_DVR_DEVICEINFO_V30(ctypes.Structure):
    _fields_ = [
        ("sSerialNumber", c_byte * 48),  # 序列号
        ("byAlarmInPortNum", c_byte),  # 模拟报警输入个数
        ("byAlarmOutPortNum", c_byte),  # 模拟报警输出个数
        ("byDiskNum", c_byte),  # 硬盘个数
        ("byDVRType", c_byte),  # 设备类型
        ("byChanNum", c_byte),  # 设备模拟通道个数，数字（IP）通道最大个数为byIPChanNum + byHighDChanNum*256
        ("byStartChan", c_byte),  # 模拟通道的起始通道号，从1开始。数字通道的起始通道号见下面参数byStartDChan
        ("byAudioChanNum", c_byte),  # 设备语音对讲通道数
        ("byIPChanNum", c_byte),  # 设备最大数字通道个数，低8位，高8位见byHighDChanNum
        ("byZeroChanNum", c_byte),  # 零通道编码个数
        ("byMainProto", c_byte),  # 主码流传输协议类型：0- private，1- rtsp，2- 同时支持私有协议和rtsp协议取流（默认采用私有协议取流）
        ("bySubProto", c_byte),  # 子码流传输协议类型：0- private，1- rtsp，2- 同时支持私有协议和rtsp协议取流（默认采用私有协议取流）
        ("bySupport", c_byte),  # 能力，位与结果为0表示不支持，1表示支持
        # bySupport & 0x1，表示是否支持智能搜索
        # bySupport & 0x2，表示是否支持备份
        # bySupport & 0x4，表示是否支持压缩参数能力获取
        # bySupport & 0x8, 表示是否支持双网卡
        # bySupport & 0x10, 表示支持远程SADP
        # bySupport & 0x20, 表示支持Raid卡功能
        # bySupport & 0x40, 表示支持IPSAN目录查找
        # bySupport & 0x80, 表示支持rtp over rtsp
        ("bySupport1", c_byte),  # 能力集扩充，位与结果为0表示不支持，1表示支持
        # bySupport1 & 0x1, 表示是否支持snmp v30
        # bySupport1 & 0x2, 表示是否支持区分回放和下载
        # bySupport1 & 0x4, 表示是否支持布防优先级
        # bySupport1 & 0x8, 表示智能设备是否支持布防时间段扩展
        # bySupport1 & 0x10,表示是否支持多磁盘数（超过33个）
        # bySupport1 & 0x20,表示是否支持rtsp over http
        # bySupport1 & 0x80,表示是否支持车牌新报警信息，且还表示是否支持NET_DVR_IPPARACFG_V40配置
        ("bySupport2", c_byte),  # 能力集扩充，位与结果为0表示不支持，1表示支持
        # bySupport2 & 0x1, 表示解码器是否支持通过URL取流解码
        # bySupport2 & 0x2, 表示是否支持FTPV40
        # bySupport2 & 0x4, 表示是否支持ANR(断网录像)
        # bySupport2 & 0x20, 表示是否支持单独获取设备状态子项
        # bySupport2 & 0x40, 表示是否是码流加密设备
        ("wDevType", c_uint16),  # 设备型号，详见下文列表
        ("bySupport3", c_byte),  # 能力集扩展，位与结果：0- 不支持，1- 支持
        # bySupport3 & 0x1, 表示是否支持多码流
        # bySupport3 & 0x4, 表示是否支持按组配置，具体包含通道图像参数、报警输入参数、IP报警输入/输出接入参数、用户参数、设备工作状态、JPEG抓图、定时和时间抓图、硬盘盘组管理等
        # bySupport3 & 0x20, 表示是否支持通过DDNS域名解析取流
        ("byMultiStreamProto", c_byte),  # 是否支持多码流，按位表示，位与结果：0-不支持，1-支持
        # byMultiStreamProto & 0x1, 表示是否支持码流3
        # byMultiStreamProto & 0x2, 表示是否支持码流4
        # byMultiStreamProto & 0x40,表示是否支持主码流
        # byMultiStreamProto & 0x80,表示是否支持子码流
        ("byStartDChan", c_byte),  # 起始数字通道号，0表示无数字通道，比如DVR或IPC
        ("byStartDTalkChan", c_byte),  # 起始数字对讲通道号，区别于模拟对讲通道号，0表示无数字对讲通道
        ("byHighDChanNum", c_byte),  # 数字通道个数，高8位
        ("bySupport4", c_byte),  # 能力集扩展，按位表示，位与结果：0- 不支持，1- 支持
        # bySupport4 & 0x01, 表示是否所有码流类型同时支持RTSP和私有协议
        # bySupport4 & 0x10, 表示是否支持域名方式挂载网络硬盘
        ("byLanguageType", c_byte),  # 支持语种能力，按位表示，位与结果：0- 不支持，1- 支持
        # byLanguageType ==0，表示老设备，不支持该字段
        # byLanguageType & 0x1，表示是否支持中文
        # byLanguageType & 0x2，表示是否支持英文
        ("byVoiceInChanNum", c_byte),  # 音频输入通道数
        ("byStartVoiceInChanNo", c_byte),  # 音频输入起始通道号，0表示无效
        ("bySupport5", c_byte ),  # 按位表示,0-不支持,1-支持,bit0-支持多码流
        ("bySupport6", c_byte),  # 按位表示,0-不支持,1-支持
        # bySupport6 & 0x1  表示设备是否支持压缩
        # bySupport6 & 0x2  表示是否支持流ID方式配置流来源扩展命令，DVR_SET_STREAM_SRC_INFO_V40
        # bySupport6 & 0x4  表示是否支持事件搜索V40接口
        # bySupport6 & 0x8  表示是否支持扩展智能侦测配置命令
        # bySupport6 & 0x40 表示图片查询结果V40扩展
        ("byMirrorChanNum", c_byte),  # 镜像通道个数，录播主机中用于表示导播通道
        ("wStartMirrorChanNo", c_uint16),  # 起始镜像通道号
        ("bySupport7", c_byte),  # 能力,按位表示,0-不支持,1-支持
        # bySupport7 & 0x1  表示设备是否支持NET_VCA_RULECFG_V42扩展
        # bySupport7 & 0x2  表示设备是否支持IPC HVT 模式扩展
        # bySupport7 & 0x04 表示设备是否支持返回锁定时间
        # bySupport7 & 0x08 表示设置云台PTZ位置时，是否支持带通道号
        # bySupport7 & 0x10 表示设备是否支持双系统升级备份
        # bySupport7 & 0x20 表示设备是否支持OSD字符叠加V50
        # bySupport7 & 0x40 表示设备是否支持主从跟踪（从摄像机）
        # bySupport7 & 0x80 表示设备是否支持报文加密
        ("byRes2", c_byte)]  # 保留，置为0
LPNET_DVR_DEVICEINFO_V30 = POINTER(NET_DVR_DEVICEINFO_V30)

# 设备参数结构体 V40
class NET_DVR_DEVICEINFO_V40(ctypes.Structure):
    _fields_ = [
        ('struDeviceV30', NET_DVR_DEVICEINFO_V30),  # 设备信息
        ('bySupportLock', c_byte),  # 设备支持锁定功能，该字段由SDK根据设备返回值来赋值的。bySupportLock为1时，dwSurplusLockTime和byRetryLoginTime有效
        ('byRetryLoginTime', c_byte),  # 剩余可尝试登陆的次数，用户名，密码错误时，此参数有效
        ('byPasswordLevel', c_byte),  # admin密码安全等级
        ('byProxyType', c_byte),  # 代理类型，0-不使用代理, 1-使用socks5代理, 2-使用EHome代理
        ('dwSurplusLockTime', c_uint32),  # 剩余时间，单位秒，用户锁定时，此参数有效
        ('byCharEncodeType', c_byte),  # 字符编码类型
        ('bySupportDev5', c_byte),  # 支持v50版本的设备参数获取，设备名称和设备类型名称长度扩展为64字节
        ('bySupport', c_byte),   # 能力集扩展，位与结果：0- 不支持，1- 支持
        ('byLoginMode', c_byte),  # 登录模式:0- Private登录，1- ISAPI登录
        ('dwOEMCode', c_uint32),  # OEM Code
        ('iResidualValidity', c_uint32),  # 该用户密码剩余有效天数，单位：天，返回负值，表示密码已经超期使用，例如“-3表示密码已经超期使用3天”
        ('byResidualValidity', c_byte),  # iResidualValidity字段是否有效，0-无效，1-有效
        ('bySingleStartDTalkChan', c_byte),  # 独立音轨接入的设备，起始接入通道号，0-为保留字节，无实际含义，音轨通道号不能从0开始
        ('bySingleDTalkChanNums', c_byte),  # 独立音轨接入的设备的通道总数，0-表示不支持
        ('byPassWordResetLevel', c_byte),  # 0-无效，
        # 1- 管理员创建一个非管理员用户为其设置密码，该非管理员用户正确登录设备后要提示“请修改初始登录密码”，未修改的情况下，用户每次登入都会进行提醒；
        # 2- 当非管理员用户的密码被管理员修改，该非管理员用户再次正确登录设备后，需要提示“请重新设置登录密码”，未修改的情况下，用户每次登入都会进行提醒。
        ('bySupportStreamEncrypt', c_byte),  # 能力集扩展，位与结果：0- 不支持，1- 支持
        # bySupportStreamEncrypt & 0x1 表示是否支持RTP/TLS取流
        # bySupportStreamEncrypt & 0x2 表示是否支持SRTP/UDP取流
        # bySupportStreamEncrypt & 0x4 表示是否支持SRTP/MULTICAST取流
        ('byMarketType', c_byte),  # 0-无效（未知类型）,1-经销型，2-行业型
        ('byRes2', c_byte * 238)  #保留，置为0
    ]
LPNET_DVR_DEVICEINFO_V40 = POINTER(NET_DVR_DEVICEINFO_V40)

# 异步登录回调函数
fLoginResultCallBack = CFUNCTYPE(None, c_uint32, c_uint32, LPNET_DVR_DEVICEINFO_V30, c_void_p)

# NET_DVR_Login_V40()参数
class NET_DVR_USER_LOGIN_INFO(Structure):
    _fields_ = [
        ("sDeviceAddress", c_char * 129),  # 设备地址，IP 或者普通域名
        ("byUseTransport", c_byte),  # 是否启用能力集透传：0- 不启用透传，默认；1- 启用透传
        ("wPort", c_uint16),  # 设备端口号，例如：8000
        ("sUserName", c_char * 64),  # 登录用户名，例如：admin
        ("sPassword", c_char * 64),  # 登录密码，例如：12345
        ("cbLoginResult", fLoginResultCallBack),  # 登录状态回调函数，bUseAsynLogin 为1时有效
        ("pUser", c_void_p),  # 用户数据
        ("bUseAsynLogin", c_uint32),  # 是否异步登录：0- 否，1- 是
        ("byProxyType", c_byte),  # 0:不使用代理，1：使用标准代理，2：使用EHome代理
        ("byUseUTCTime", c_byte),
        # 0-不进行转换，默认,1-接口上输入输出全部使用UTC时间,SDK完成UTC时间与设备时区的转换,2-接口上输入输出全部使用平台本地时间，SDK完成平台本地时间与设备时区的转换
        ("byLoginMode", c_byte),  # 0-Private 1-ISAPI 2-自适应
        ("byHttps", c_byte),  # 0-不适用tls，1-使用tls 2-自适应
        ("iProxyID", c_uint32),  # 代理服务器序号，添加代理服务器信息时，相对应的服务器数组下表值
        ("byVerifyMode", c_byte),  # 认证方式，0-不认证，1-双向认证，2-单向认证；认证仅在使用TLS的时候生效;
        ("byRes2", c_byte * 119)]
LPNET_DVR_USER_LOGIN_INFO = POINTER(NET_DVR_USER_LOGIN_INFO)

# 组件库加载路径信息
class NET_DVR_LOCAL_SDK_PATH(Structure):
    pass
LPNET_DVR_LOCAL_SDK_PATH = POINTER(NET_DVR_LOCAL_SDK_PATH)
NET_DVR_LOCAL_SDK_PATH._fields_ = [
    ('sPath', c_char * 256),  # 组件库地址
    ('byRes', c_byte * 128),
]

# 定义预览参数结构体
class NET_DVR_PREVIEWINFO(Structure):
    pass
LPNET_DVR_PREVIEWINFO = POINTER(NET_DVR_PREVIEWINFO)
NET_DVR_PREVIEWINFO._fields_ = [
    ('lChannel', c_uint32),  # 通道号
    ('dwStreamType', c_uint32),  # 码流类型，0-主码流，1-子码流，2-码流3，3-码流4, 4-码流5,5-码流6,7-码流7,8-码流8,9-码流9,10-码流10
    ('dwLinkMode', c_uint32),  # 0：TCP方式,1：UDP方式,2：多播方式,3 - RTP方式，4-RTP/RTSP,5-RSTP/HTTP ,6- HRUDP（可靠传输） ,7-RTSP/HTTPS
    ('hPlayWnd', c_uint32),  # 播放窗口的句柄,为NULL表示不播放图象
    ('bBlocked', c_uint32),  # 0-非阻塞取流, 1-阻塞取流, 如果阻塞SDK内部connect失败将会有5s的超时才能够返回,不适合于轮询取流操作
    ('bPassbackRecord', c_uint32),  # 0-不启用录像回传,1启用录像回传
    ('byPreviewMode', c_ubyte),  # 预览模式，0-正常预览，1-延迟预览
    ('byStreamID', c_ubyte * 32),  # 流ID，lChannel为0xffffffff时启用此参数
    ('byProtoType', c_ubyte),  # 应用层取流协议，0-私有协议，1-RTSP协议,
    # 2-SRTP码流加密（对应此结构体中dwLinkMode 字段，支持如下方式, 为1，表示udp传输方式，信令走TLS加密，码流走SRTP加密，为2，表示多播传输方式，信令走TLS加密，码流走SRTP加密）
    ('byRes1', c_ubyte),
    ('byVideoCodingType', c_ubyte),  # 码流数据编解码类型 0-通用编码数据 1-热成像探测器产生的原始数据
    ('dwDisplayBufNum', c_uint32),  # 播放库播放缓冲区最大缓冲帧数，范围1-50，置0时默认为1
    ('byNPQMode', c_ubyte),  # NPQ是直连模式，还是过流媒体：0-直连 1-过流媒体
    ('byRecvMetaData', c_ubyte),  # 是否接收metadata数据
    # 设备是否支持该功能通过GET /ISAPI/System/capabilities 中DeviceCap.SysCap.isSupportMetadata是否存在且为true
    ('byDataType', c_ubyte),  # 数据类型，0-码流数据，1-音频数据
    ('byRes', c_ubyte * 213),
]

#定义JPEG图像信息结构体
class NET_DVR_JPEGPARA(Structure):
    pass
LPNET_DVR_JPEGPARA = POINTER(NET_DVR_JPEGPARA)
NET_DVR_JPEGPARA._fields_ = [
    ('wPicSize', c_ushort),
    ('wPicQuality', c_ushort),
]

# 叠加字符
class NET_DVR_SHOWSTRINGINFO(Structure):
    pass
LPNET_DVR_SHOWSTRINGINFO = POINTER(NET_DVR_SHOWSTRINGINFO)
NET_DVR_SHOWSTRINGINFO._fields_ = [
    ('wShowString', c_ushort),
    ('wStringSize', c_ushort),
    ('wShowStringTopLeftX', c_ushort),
    ('wShowStringTopLeftY', c_ushort),
    ('sString', c_ubyte * 44),
]

# 叠加字符
class NET_DVR_SHOWSTRING_V30(Structure):
    pass
LPNET_DVR_SHOWSTRING_V30 = POINTER(NET_DVR_SHOWSTRING_V30)
NET_DVR_SHOWSTRING_V30._fields_ = [
    ('dwSize', c_uint32),
    ('struStringInfo', NET_DVR_SHOWSTRINGINFO * 8),
]

# 透传接口输出参数结构体
class NET_DVR_XML_CONFIG_OUTPUT(Structure):
    pass
LPNET_DVR_XML_CONFIG_OUTPUT = POINTER(NET_DVR_XML_CONFIG_OUTPUT)
NET_DVR_XML_CONFIG_OUTPUT._fields_ = [
    ('dwSize', c_uint32),
    ('lpOutBuffer', c_void_p),
    ('dwOutBufferSize', c_uint32),
    ('dwReturnedXMLSize', c_uint32),
    ('lpStatusBuffer', c_void_p),
    ('dwStatusSize', c_uint32),
    ('byRes', c_ubyte * 32)
]

# 透传接口输入参数结构体
class NET_DVR_XML_CONFIG_INPUT(Structure):
    pass
LPNET_DVR_XML_CONFIG_INPUT = POINTER(NET_DVR_XML_CONFIG_INPUT)
NET_DVR_XML_CONFIG_INPUT._fields_ = [
    ('dwSize', c_uint32),
    ('lpRequestUrl', c_void_p),
    ('dwRequestUrlLen', c_uint32),
    ('lpInBuffer', c_void_p),
    ('dwInBufferSize', c_uint32),
    ('dwRecvTimeOut', c_uint32),
    ('byForceEncrpt', c_ubyte),
    ('byNumOfMultiPart', c_ubyte),
    ('byRes', c_ubyte * 30)
]

# 报警设备信息结构体
class NET_DVR_ALARMER(Structure):
    _fields_ = [
        ("byUserIDValid", c_byte),  # UserID是否有效 0-无效，1-有效
        ("bySerialValid", c_byte),  # 序列号是否有效 0-无效，1-有效
        ("byVersionValid", c_byte),  # 版本号是否有效 0-无效，1-有效
        ("byDeviceNameValid", c_byte),  # 设备名字是否有效 0-无效，1-有效
        ("byMacAddrValid", c_byte),  # MAC地址是否有效 0-无效，1-有效
        ("byLinkPortValid", c_byte),  # login端口是否有效 0-无效，1-有效
        ("byDeviceIPValid", c_byte),  # 设备IP是否有效 0-无效，1-有效
        ("bySocketIPValid", c_byte),  # socket ip是否有效 0-无效，1-有效
        ("lUserID", c_uint32),  # NET_DVR_Login()返回值, 布防时有效
        ("sSerialNumber", c_byte * 48),  # 序列号
        ("dwDeviceVersion", c_uint32),  # 版本信息 高16位表示主版本，低16位表示次版本
        ("sDeviceName", c_byte * 32),  # 设备名字
        ("byMacAddr", c_byte * 6),  # MAC地址
        ("wLinkPort", c_uint16),  # link port
        ("sDeviceIP", c_byte * 128),  # IP地址
        ("sSocketIP", c_byte * 128),  # 报警主动上传时的socket IP地址
        ("byIpProtocol", c_byte),  # Ip协议 0-IPV4, 1-IPV6
        ("byRes2", c_byte * 11)]
LPNET_DVR_ALARMER = POINTER(NET_DVR_ALARMER)

# 报警布防参数结构体
class NET_DVR_SETUPALARM_PARAM(Structure):
    _fields_ = [
        ("dwSize", c_uint32),  # 结构体大小
        ("byLevel", c_byte),  # 布防优先级：0- 一等级（高），1- 二等级（中），2- 三等级（低）
        ("byAlarmInfoType", c_byte),
        # 上传报警信息类型（抓拍机支持），0-老报警信息（NET_DVR_PLATE_RESULT），1-新报警信息(NET_ITS_PLATE_RESULT)2012-9-28
        ("byRetAlarmTypeV40", c_byte),
        # 0- 返回NET_DVR_ALARMINFO_V30或NET_DVR_ALARMINFO,
        # 1- 设备支持NET_DVR_ALARMINFO_V40则返回NET_DVR_ALARMINFO_V40，不支持则返回NET_DVR_ALARMINFO_V30或NET_DVR_ALARMINFO
        ("byRetDevInfoVersion", c_byte),  # CVR上传报警信息回调结构体版本号 0-COMM_ALARM_DEVICE， 1-COMM_ALARM_DEVICE_V40
        ("byRetVQDAlarmType", c_byte),  # VQD报警上传类型，0-上传报报警NET_DVR_VQD_DIAGNOSE_INFO，1-上传报警NET_DVR_VQD_ALARM
        ("byFaceAlarmDetection", c_byte),
        ("bySupport", c_byte),
        ("byBrokenNetHttp", c_byte),
        ("wTaskNo", c_uint16),
        # 任务处理号 和 (上传数据NET_DVR_VEHICLE_RECOG_RESULT中的字段dwTaskNo对应 同时 下发任务结构 NET_DVR_VEHICLE_RECOG_COND中的字段dwTaskNo对应)
        ("byDeployType", c_byte),  # 布防类型：0-客户端布防，1-实时布防
        ("byRes1", c_byte * 3),
        ("byAlarmTypeURL", c_byte),
        # bit0-表示人脸抓拍报警上传
        # 0-表示二进制传输，1-表示URL传输（设备支持的情况下，设备支持能力根据具体报警能力集判断,同时设备需要支持URL的相关服务，当前是”云存储“）
        ("byCustomCtrl", c_byte)]  # Bit0- 表示支持副驾驶人脸子图上传: 0-不上传,1-上传
LPNET_DVR_SETUPALARM_PARAM = POINTER(NET_DVR_SETUPALARM_PARAM)

# 上传的报警信息结构体。
class NET_DVR_ALARMINFO_V30(Structure):
    _fields_ = [
        ("dwAlarmType", c_uint32),  # 报警类型
        ("dwAlarmInputNumber", c_uint32),  # 报警输入端口，当报警类型为0、23时有效
        ("byAlarmOutputNumber", c_byte * 96),
        # 触发的报警输出端口，值为1表示该报警端口输出，如byAlarmOutputNumber[0]=1表示触发第1个报警输出口输出，byAlarmOutputNumber[1]=1表示触发第2个报警输出口，依次类推
        ("byAlarmRelateChannel", c_byte * 64),  # 触发的录像通道，值为1表示该通道录像，如byAlarmRelateChannel[0]=1表示触发第1个通道录像
        ("byChannel", c_byte * 64),  # 发生报警的通道。当报警类型为2、3、6、9、10、11、13、15、16时有效，如byChannel[0]=1表示第1个通道报警
        ("byDiskNumber", c_byte * 33)]  # 发生报警的硬盘。当报警类型为1，4，5时有效，byDiskNumber[0]=1表示1号硬盘异常
LPNET_DVR_ALARMINFO_V30 = POINTER(NET_DVR_ALARMINFO_V30)


# 报警布防参数结构体
class NET_DVR_SETUPALARM_PARAM(Structure):
    _fields_ = [
        ("dwSize", c_uint32),  # 结构体大小
        ("byLevel", c_byte),  # 布防优先级：0- 一等级（高），1- 二等级（中），2- 三等级（低）
        ("byAlarmInfoType", c_byte),
        # 上传报警信息类型（抓拍机支持），0-老报警信息（NET_DVR_PLATE_RESULT），1-新报警信息(NET_ITS_PLATE_RESULT)2012-9-28
        ("byRetAlarmTypeV40", c_byte),
        # 0- 返回NET_DVR_ALARMINFO_V30或NET_DVR_ALARMINFO,
        # 1- 设备支持NET_DVR_ALARMINFO_V40则返回NET_DVR_ALARMINFO_V40，不支持则返回NET_DVR_ALARMINFO_V30或NET_DVR_ALARMINFO
        ("byRetDevInfoVersion", c_byte),  # CVR上传报警信息回调结构体版本号 0-COMM_ALARM_DEVICE， 1-COMM_ALARM_DEVICE_V40
        ("byRetVQDAlarmType", c_byte),  # VQD报警上传类型，0-上传报报警NET_DVR_VQD_DIAGNOSE_INFO，1-上传报警NET_DVR_VQD_ALARM
        ("byFaceAlarmDetection", c_byte),
        ("bySupport", c_byte),
        ("byBrokenNetHttp", c_byte),
        ("wTaskNo", c_uint16),
        # 任务处理号 和 (上传数据NET_DVR_VEHICLE_RECOG_RESULT中的字段dwTaskNo对应 同时 下发任务结构 NET_DVR_VEHICLE_RECOG_COND中的字段dwTaskNo对应)
        ("byDeployType", c_byte),  # 布防类型：0-客户端布防，1-实时布防
        ("byRes1", c_byte * 3),
        ("byAlarmTypeURL", c_byte),
        # bit0-表示人脸抓拍报警上传
        # 0- 表示二进制传输，1- 表示URL传输（设备支持的情况下，设备支持能力根据具体报警能力集判断,同时设备需要支持URL的相关服务，当前是”云存储“）
        ("byCustomCtrl", c_byte)] # Bit0- 表示支持副驾驶人脸子图上传: 0-不上传,1-上传,(注：只在公司内部8600/8200等平台开放)
LPNET_DVR_SETUPALARM_PARAM = POINTER(NET_DVR_SETUPALARM_PARAM)


# 时间参数结构体
class NET_DVR_TIME(Structure):
    _fields_ = [
        ("dwYear", c_uint32),  # 年
        ("dwMonth", c_uint32),  # 月
        ("dwDay", c_uint32),  # 日
        ("dwHour", c_uint32),  # 时
        ("dwMinute", c_uint32),  # 分
        ("dwSecond", c_uint32)]  # 秒
LPNET_DVR_TIME = POINTER(NET_DVR_TIME)


# IP地址结构体
class NET_DVR_IPADDR(Structure):
    _fields_ = [
        ("sIpV4", c_byte * 16),  # 设备IPv4地址
        ("sIpV6", c_byte * 128)]  # 设备IPv6地址
LPNET_DVR_IPADDR = POINTER(NET_DVR_IPADDR)


# 门禁主机事件信息
class NET_DVR_ACS_EVENT_INFO(Structure):
    _fields_ = [
        ("dwSize", c_uint32),  # 结构体大小
        ("byCardNo", c_byte * 32),  # 卡号
        ("byCardType", c_byte),  # 卡类型：1- 普通卡，2- 残障人士卡，3- 黑名单卡，4- 巡更卡，5- 胁迫卡，6- 超级卡，7- 来宾卡，8- 解除卡，为0表示无效
        ("byAllowListNo", c_byte),  # 白名单单号，取值范围：1~8，0表示无效
        ("byReportChannel", c_byte),  # 报告上传通道：1- 布防上传，2- 中心组1上传，3- 中心组2上传，0表示无效
        ("byCardReaderKind", c_byte),  # 读卡器类型：0- 无效，1- IC读卡器，2- 身份证读卡器，3- 二维码读卡器，4- 指纹头
        ("dwCardReaderNo", c_uint32),  # 读卡器编号，为0表示无效
        ("dwDoorNo", c_uint32),  # 门编号（或者梯控的楼层编号），为0表示无效（当接的设备为人员通道设备时，门1为进方向，门2为出方向）
        ("dwVerifyNo", c_uint32),  # 多重卡认证序号，为0表示无效
        ("dwAlarmInNo", c_uint32),  # 报警输入号，为0表示无效
        ("dwAlarmOutNo", c_uint32),  # 报警输出号，为0表示无效
        ("dwCaseSensorNo", c_uint32),  # 事件触发器编号
        ("dwRs485No", c_uint32),  # RS485通道号，为0表示无效
        ("dwMultiCardGroupNo", c_uint32),  # 群组编号
        ("wAccessChannel", c_uint16),  # 人员通道号
        ("byDeviceNo", c_byte),  # 设备编号，为0表示无效
        ("byDistractControlNo", c_byte),  # 分控器编号，为0表示无效
        ("dwEmployeeNo", c_uint32),  # 工号，为0无效
        ("wLocalControllerID", c_uint16),  # 就地控制器编号，0-门禁主机，1-255代表就地控制器
        ("byInternetAccess", c_byte),  # 网口ID：（1-上行网口1,2-上行网口2,3-下行网口1）
        ("byType", c_byte),
        # 防区类型，0:即时防区,1-24小时防区,2-延时防区,3-内部防区,4-钥匙防区,5-火警防区,6-周界防区,7-24小时无声防区,
        # 8-24小时辅助防区,9-24小时震动防区,10-门禁紧急开门防区,11-门禁紧急关门防区，0xff-无
        ("byMACAddr", c_byte * 6),  # 物理地址，为0无效
        ("bySwipeCardType", c_byte),  # 刷卡类型，0-无效，1-二维码
        ("byMask", c_byte),  # 是否带口罩：0-保留，1-未知，2-不戴口罩，3-戴口罩
        ("dwSerialNo", c_uint32),  # 事件流水号，为0无效
        ("byChannelControllerID", c_byte),  # 通道控制器ID，为0无效，1-主通道控制器，2-从通道控制器
        ("byChannelControllerLampID", c_byte),  # 通道控制器灯板ID，为0无效（有效范围1-255）
        ("byChannelControllerIRAdaptorID", c_byte),  # 通道控制器红外转接板ID，为0无效（有效范围1-255）
        ("byChannelControllerIREmitterID", c_byte),  # 通道控制器红外对射ID，为0无效（有效范围1-255）
        ("byHelmet", c_byte),  # 可选，是否戴安全帽：0-保留，1-未知，2-不戴安全, 3-戴安全帽
        ("byRes", c_byte * 3)]  # 保留，置为0
LPNET_DVR_ACS_EVENT_INFO = POINTER(NET_DVR_ACS_EVENT_INFO)


# 门禁主机报警信息结构体
class NET_DVR_ACS_ALARM_INFO(Structure):
    _fields_ = [
        ("dwSize", c_uint32),  # 结构体大小
        ("dwMajor", c_uint32),  # 报警主类型，具体定义见“Remarks”说明
        ("dwMinor", c_uint32),  # 报警次类型，次类型含义根据主类型不同而不同，具体定义见“Remarks”说明
        ("struTime", NET_DVR_TIME),  # 报警时间
        ("sNetUser", c_byte * 16),  # 网络操作的用户名
        ("struRemoteHostAddr", NET_DVR_IPADDR),  # 远程主机地址
        ("struAcsEventInfo", NET_DVR_ACS_EVENT_INFO),  # 报警信息详细参数
        ("dwPicDataLen", c_uint32),  # 图片数据大小，不为0是表示后面带数据
        ("pPicData", c_void_p),  # 图片数据缓冲区
        ("wInductiveEventType", c_uint16),  # 归纳事件类型，0-无效，客户端判断该值为非0值后，报警类型通过归纳事件类型区分，否则通过原有报警主次类型（dwMajor、dwMinor）区分
        ("byPicTransType", c_byte),  # 图片数据传输方式: 0-二进制；1-url
        ("byRes1", c_byte),  # 保留，置为0
        ("dwIOTChannelNo", c_uint32),  # IOT通道号
        ("pAcsEventInfoExtend", c_void_p),  # byAcsEventInfoExtend为1时，表示指向一个NET_DVR_ACS_EVENT_INFO_EXTEND结构体
        ("byAcsEventInfoExtend", c_byte),  # pAcsEventInfoExtend是否有效：0-无效，1-有效
        ("byTimeType", c_byte),  # 时间类型：0-设备本地时间，1-UTC时间（struTime的时间）
        ("byRes2", c_byte),  # 保留，置为0
        ("byAcsEventInfoExtendV20", c_byte),  # pAcsEventInfoExtendV20是否有效：0-无效，1-有效
        ("pAcsEventInfoExtendV20", c_void_p),  # byAcsEventInfoExtendV20为1时，表示指向一个NET_DVR_ACS_EVENT_INFO_EXTEND_V20结构体
        ("byRes", c_byte * 4)]  # 保留，置为0
LPNET_DVR_ACS_ALARM_INFO = POINTER(NET_DVR_ACS_ALARM_INFO)

# 点坐标参数结构体
class NET_VCA_POINT(Structure):
    _fields_ = [
        ("fX", c_float),
        ("fY", c_float)
]

# 身份证刷卡信息扩展参数
class NET_DVR_ID_CARD_INFO_EXTEND(Structure):
    _fields_ = [
        ("byRemoteCheck", c_ubyte),
        ("byThermometryUnit", c_ubyte),
        ("byIsAbnomalTemperature", c_ubyte),
        ("byRes2", c_ubyte),
        ("fCurrTemperature", c_float),
        ("struRegionCoordinates", NET_VCA_POINT),
        ("dwQRCodeInfoLen",c_uint32),
        ("dwVisibleLightDataLen",c_uint32),
        ("dwThermalDataLen",c_uint32),
        ("pQRCodeInfo", POINTER(c_byte)),
        ("pVisibleLightData", POINTER(c_byte)),
        ("pThermalData", POINTER(c_byte)),
        ("byRes", c_ubyte * 1024)
]

# 日期信息结构体
class NET_DVR_DATE(Structure):
    _fields_ = [
    ('wYear', c_ushort),
    ('byMonth', c_ubyte),
    ('byDay', c_ubyte)
]

# 身份证信息结构体
class NET_DVR_ID_CARD_INFO(Structure):
    _fields_ = [
        ("dwSize",c_uint),
        ("byName", c_ubyte * 128),
        ("struBirth",NET_DVR_DATE),
        ("byAddr", c_ubyte * 280),
        ("byIDNum", c_ubyte * 32),
        ("byIssuingAuthority", c_ubyte * 128),
        ("struStartDate",NET_DVR_DATE),
        ("struEndDate",NET_DVR_DATE),
        ("byTermOfValidity", c_ubyte),
        ("bySex", c_ubyte),
        ("byNation", c_ubyte),
        ("byRes", c_ubyte * 101)
]

# 时间参数结构体
class NET_DVR_TIME(Structure):
    _fields_ = [
        ("dwYear", c_uint32),
        ("dwMonth", c_uint32),
        ("dwDay", c_uint32),
        ("dwHour", c_uint32),
        ("dwMinute", c_uint32),
        ("dwSecond", c_uint32)
]

# 时间参数结构体
class NET_DVR_TIME_V30(Structure):
    _fields_ = [
    ('wYear', c_ushort),
    ('byMonth', c_ubyte),
    ('byDay', c_ubyte),
    ('byHour', c_ubyte),
    ('byMinute', c_ubyte),
    ('bySecond', c_ubyte),
    ('byISO8601', c_ubyte),
    ('wMilliSec', c_ushort),
    ('cTimeDifferenceH', c_ubyte),
    ('cTimeDifferenceM', c_ubyte),
]

# IP地址结构体
class NET_DVR_IPADDR(Structure):
    _fields_ = [
        ("sIpV4", c_ubyte * 16),
        ("byIPv6", c_ubyte * 128)]

# 身份证刷卡信息上传结构体
class NET_DVR_ID_CARD_INFO_ALARM(Structure):
    _fields_ = [
        ("dwSize",c_uint32),  # 结构长度
        ("struIDCardCfg", NET_DVR_ID_CARD_INFO),  # 身份证信息
        ("dwMajor", c_uint32),  # 报警主类型，参考宏定义
        ("dwMinor", c_uint32),  # 报警次类型，参考宏定义
        ("struSwipeTime", NET_DVR_TIME_V30),  # 刷卡时间
        ("byNetUser", c_ubyte * 16),  # 网络操作的用户名
        ("struRemoteHostAddr", NET_DVR_IPADDR),  # 远程主机地址
        ("dwCardReaderNo", c_uint32),  # 读卡器编号，为0无效
        ("dwDoorNo", c_uint32),  # 门编号，为0无效
        ("dwPicDataLen", c_uint32),  # 图片数据大小，不为0是表示后面带数据
        ("pPicData", c_void_p),  # 身份证图片数据缓冲区，dwPicDataLen不为0时缓冲区里面存放身份证头像的图片数据
        ("byCardType", c_ubyte),  # 卡类型，1-普通卡，2-残疾人卡，3-黑名单卡，4-巡更卡，5-胁迫卡，6-超级卡，7-来宾卡，8-解除卡，为0无效
        ("byDeviceNo", c_ubyte),  # 设备编号，为0时无效（有效范围1-255）
        ("byMask", c_ubyte),  # 是否带口罩：0-保留，1-未知，2-不戴口罩，3-戴口罩
        ("byRes2", c_ubyte),  # 保留，置为0
        ("dwFingerPrintDataLen", c_uint32),  # 指纹数据大小，不为0是表示后面带数据
        ("pFingerPrintData", c_void_p),  # 指纹数据缓冲区，dwFingerPrintDataLen不为0时缓冲区里面存放指纹数据
        ("dwCapturePicDataLen", c_uint32),  # 抓拍图片数据大小，不为0是表示后面带数据
        ("pCapturePicData", c_void_p),  # 抓拍图片数据缓冲区，dwCapturePicDataLen不为0时缓冲区里面存放设备上摄像机抓拍上传的图片数据
        ("dwCertificatePicDataLen", c_uint32),  # 证件抓拍图片数据大小，不为0是表示后面带数据
        ("pCertificatePicData", c_void_p),  # 证件抓拍图片数据缓冲区，dwCertificatePicDataLen不为0时缓冲区里面存放设备上摄像机抓拍上传的证件抓拍图片数据
        ("byCardReaderKind", c_ubyte),  # 读卡器属于哪一类：0-无效，1-IC读卡器，2-身份证读卡器，3-二维码读卡器，4-指纹头
        ("byRes3", c_ubyte * 2),  # 保留，置为0
        ("byIDCardInfoExtend", c_ubyte),  # pIDCardInfoExtend是否有效：0-无效，1-有效
        ("pIDCardInfoExtend", POINTER(NET_DVR_ID_CARD_INFO_EXTEND)),  # 身份证刷卡扩展事件信息
        ("byRes", c_ubyte * 172)  # 身份证刷卡扩展事件信息
    ]
LPNET_DVR_ID_CARD_INFO_ALARM = POINTER(NET_DVR_ID_CARD_INFO_ALARM)

class NET_DVR_ALARM_ISAPI_PICDATA(Structure):
    _fields_ = [
        ("dwPicLen", c_uint32),  # 图片数据长度
        ("byPicType", c_ubyte),  # 图片格式: 1- jpg
        ("byRes", c_ubyte * 3),  #
        ("szFilename", c_ubyte * 256),  # 图片名称
        ("pPicData", c_void_p),  # 图片数据
    ]
LPNET_DVR_ALARM_ISAPI_PICDATA = POINTER(NET_DVR_ALARM_ISAPI_PICDATA)

class NET_DVR_ALARM_ISAPI_INFO(Structure):
    _fields_ = [
        ("pAlarmData", c_void_p),  # 报警数据
        ("dwAlarmDataLen", c_uint32),  # 报警数据长度
        ("byDataType", c_ubyte),  # 0-invalid,1-xml,2-json
        ("byPicturesNumber", c_ubyte),  # 图片数量
        ("byRes[2]", c_ubyte * 2),  # 保留字节
        ("pPicPackData", c_void_p),  # 图片变长部分
        ("byRes1[32]", c_ubyte * 32),  # 保留字节
    ]
LPNET_DVR_ALARM_ISAPI_INFO = POINTER(NET_DVR_ALARM_ISAPI_INFO)

class NET_DVR_LOCAL_GENERAL_CFG(Structure):
    _fields_ = [
        ("byExceptionCbDirectly", c_ubyte),  # 0-通过线程池异常回调，1-直接异常回调给上层
        ("byNotSplitRecordFile", c_ubyte),  # 回放和预览中保存到本地录像文件不切片 0-默认切片，1-不切片
        ("byResumeUpgradeEnable", c_ubyte),  # 断网续传升级使能，0-关闭（默认），1-开启
        ("byAlarmJsonPictureSeparate", c_ubyte),  # 控制JSON透传报警数据和图片是否分离，0-不分离，1-分离（分离后走COMM_ISAPI_ALARM回调返回）
        ("byRes", c_ubyte * 4),  # 保留
        ("i64FileSize", c_uint64),  # 单位：Byte
        ("dwResumeUpgradeTimeout", c_uint32),  # 断网续传重连超时时间，单位毫秒
        ("byAlarmReconnectMode", c_ubyte),  # 0-独立线程重连（默认） 1-线程池重连
        ("byStdXmlBufferSize", c_ubyte),  # 设置ISAPI透传接收缓冲区大小，1-1M 其他-默认
        ("byMultiplexing", c_ubyte),  # 0-普通链接（非TLS链接）关闭多路复用，1-普通链接（非TLS链接）开启多路复用
        ("byFastUpgrade", c_ubyte),  # 0-正常升级，1-快速升级
        ("byRes1", c_ubyte * 232),  # 预留
    ]
LPNET_DVR_LOCAL_GENERAL_CFG = POINTER(NET_DVR_LOCAL_GENERAL_CFG)


# 报警信息回调函数
MSGCallBack_V31 = fun_ctype(c_bool, c_uint32, LPNET_DVR_ALARMER, c_void_p, c_ulong, c_void_p)
MSGCallBack = fun_ctype(None, c_uint32, LPNET_DVR_ALARMER, c_void_p, c_ulong, c_void_p)
# 码流回调函数
REALDATACALLBACK = fun_ctype(None, c_long, c_ulong, POINTER(c_ubyte), c_ulong, c_void_p)



#!usr/bin/python
# _*_ coding:utf-8 _*_
"""python
Created on 2020/08/14
@author: luzhipeng
"""
import os
import time
from smb.SMBConnection import SMBConnection
from smb.smb_structs import OperationFailure

class dxr_file:
    def __init__(self, ip='10.10.10.53', port=445, user_name='admin', passwd='123456'):
        self.ip = ip
        self.port = port
        self.user_name = user_name
        self.passwd = passwd
        self.samba, self.status = self.connect()
        if self.status:
            print('连接成功')
        
        
    def connect(self):
        '''
        建立smb服务连接
        :param user_name:
        :param passwd:
        :param ip:
        :param port: 445或者139
        :return:
        '''
        try:
            samba = SMBConnection(self.user_name, self.passwd, 'dxr', 'dxr', use_ntlm_v2=True)
            samba.connect(self.ip, self.port)
            status = samba.auth_result
        except:
            samba.close()
        return samba, status
        
    def all_shares_name(self):
        '''
        列出smb服务器下的所有共享目录
        :param samba:
        :return:
        '''
        share_names = list()
        sharelist = self.samba.listShares()
        for s in sharelist:
            share_names.append(s.name)
        return share_names
    
    def all_file_names_in_dir(self, service_name, dir_name):
        '''
        列出文件夹内所有文件名
        :param service_name: 服务名（smb中的文件夹名，一级目录）
        :param dir_name: 二级目录及以下的文件目录
        :return:
        '''
        f_names = list()
        for e in self.samba.listPath(service_name, dir_name):
            if e.filename[0] != '.':   # （会返回一些.的文件，需要过滤）
                f_names.append(e.filename)
        return f_names
    
    def get_last_updatetime(self, service_name, file_path):
        '''
        返回samba server上的文件更新时间（时间戳），如果出现OperationFailure说明无此文件，返回0
        :param samba:
        :param service_name:
        :param file_path:
        :return:
        '''
        try:
            sharedfile_obj = self.samba.getAttributes(service_name, file_path)
            return sharedfile_obj.last_write_time
        except OperationFailure:
            return 0
        
    def download(self, f_names, service_name, smb_dir, local_dir):
        '''
        下载文件
        :param samba:
        :param f_names:文件名
        :param service_name:服务名（smb中的文件夹名）
        :param smb_dir: smb文件夹
        :param local_dir: 本地文件夹
        :return:
        '''
        assert isinstance(f_names, list)
        # 如果local_dir不存在，则创建
        if not os.path.exists(local_dir):
            print('本地文件夹不存在，创建文件夹', local_dir)
            os.makedirs(local_dir)
        try:
            for f_name in f_names:
                f = open(os.path.join(local_dir, f_name), 'wb')
                self.samba.retrieveFile(service_name, os.path.join(smb_dir, f_name), f)
                f.close()
            return True
        except:
            return False
        
    def createDir(self, service_name, path):
        """
        创建文件夹
        :param samba:
        :param service_name:
        :param path:
        :return:
        """
        try:
            self.samba.createDirectory(service_name, path)
            return True
        except OperationFailure:
            return False

    def upload(self, service_name, smb_dir, local_dir, f_name):
        '''
        上传文件
        :param samba:
        :param service_name:服务名（smb中的文件夹名）
        :param smb_dir: smb文件夹
        :param local_dir: 本地文件列表所在目录
        :param f_name: 本地文件名
        :return: 返回http://ip:port/服务名/文件夹名/文件名
        '''
        try:
            f = open(os.path.join(local_dir, f_name), 'rb')
            shares_list = self.samba.listShares()
            # 如果smb_dir不存在，则创建
            # 如果smb_dir中包含/，则需要创建多级目录
            if smb_dir.count('/') > 0:
                smb_dir_list = smb_dir.split('/')
                for i in range(len(smb_dir_list)):
                    if smb_dir_list[i] not in self.all_file_names_in_dir(service_name, '/'.join(smb_dir_list[:i])):
                        self.createDir(service_name, '/'.join(smb_dir_list[:i+1]))
                        print('创建目录：', '/'.join(smb_dir_list[:i+1]))
            self.samba.storeFile(service_name, os.path.join(smb_dir, f_name), f)
            f.close()
            return 'http://{}:{}/{}/{}'.format(self.ip, 880, smb_dir, f_name)
        except Exception as e:
            print(e)
            return ''
        
if __name__ == '__main__':
    smb = dxr_file('10.10.10.53', 445, 'admin', '123456')
    # 上传本文件夹下的lu.wav文件
    for i in range(10):
        print(smb.upload('dxr', 'test/test', os.path.dirname(__file__), 'test.png'))
        # 下载test/test文件夹下的lu.wav文件
        current_dir = os.path.dirname(__file__)
        time.sleep(1)
        print(smb.download(['lu.wav'], 'dxr', 'test/test', current_dir.join('test')))


from setuptools import setup

setup(
    name='DXR',
    version='1.3.8',
    packages=['Dxr_mqtt', 'Dxr_log', 'Dxr_bytes', 'Dxr_utils', 'Dxr_video', 'Dxr_serial', 'Dxr_yaml', 'Dxr_file'],
    install_requires=['paho-mqtt', 'pyyaml', 'pyserial', 'loguru','tabulate', 'pymysql'],
    author='luzhipeng',
    author_email='402087139@qq.com',
    license='MIT',
    url='http://pycn.me',
    description='DXR is a python library for DXR_mqtt',
)



from queue import Queue

ip = "192.168.10.100"
isStart = False
server_isStart = False
push_server_isStart = False
isStart_from_url = False
url = 0
port = 5001
video_que = Queue(maxsize=1)
video_from_url_que = Queue(maxsize=1)
hik_ip = '192.168.1.64'
hik_password = 'Asb11023'
hik_username = 'admin'

